#ifndef _OOS_SDK_C_API_DEFINE_H_
#define _OOS_SDK_C_API_DEFINE_H_
#include "oos_string.h"

typedef struct request_options_t{
    oos_string_t endpoint;
    oos_string_t access_key;
    oos_string_t secret_key;
    // 使用签名版本 OOS_FLASE v2, OOS_TRUE v4
    oos_bool is_v4;
    // 是否负载进行签名，只有v4版本有效 OOS_TRUE 签名 OOS_FLASE 不签名
    oos_bool is_sign_payload;
    oos_pool_t *pool;
}request_options_t;

/**
 * @brief The DataLocationType enum
 * LOCAL:  本地
 * SPECIFIED: 指定
 */
enum dataLocationType_e{LOCAL, SPECIFIED};

/**
 * @brief The scheduleStrategy_e enum
 * ALLOWED: 允许
 * NOTALLOWED:  不允许
 */
enum scheduleStrategy_e{ALLOWED, NOTALLOWED};

/**
 * list_string_node_t
 */
typedef struct list_string_node_t{
    struct list_head node;
    oos_string_t text;
}list_string_node_t;

/**
 * bucket location
 */
typedef struct location_configuration_t{
    // medadata location
    oos_string_t medadata_location;
    // data location
    enum dataLocationType_e data_location_type;
    // scheduleStrategy
    enum scheduleStrategy_e data_schedule_strategy;
    // data location list
    list_string_node_t data_location_list;

} location_configuration_t;

/**
 *  region_configuration_t
 */
typedef struct region_configuration_t{
    // meta_regions_list
    list_string_node_t meta_location_list;
    // data_regions_list
    list_string_node_t data_location_list;
} region_configuration_t;

/**
 * list_delete_node_t
 */
typedef struct list_delete_node_t{
    struct list_head node;
    oos_string_t object_key;
    oos_string_t result;
}list_delete_node_t;

/**
 *  delete_multiple_configuration_t
 */
typedef struct delete_multiple_list_t{
    // delete_object_list
    list_delete_node_t delete_list;
} delete_multiple_list_t;

/**
 * @brief The accelerate_status_e enum
 * ENABLED: 启用
 * SUSPENDED:  挂起
 */
enum accelerate_status_e{ENABLED, SUSPENDED};


typedef struct acl_grant_info{
    struct list_head node;
    oos_string_t grantee_uri;
    oos_string_t permission;
}acl_grant_info;

/**
 *  bucket acl
 */
typedef struct acl_configuration_t{
    oos_string_t id;
    oos_string_t display_name;
    acl_grant_info grant_list;
} acl_configuration_t;


/**
 *  bucket accelerater
 */
typedef struct accelerate_configuration_t{
    // accelerate status
    enum accelerate_status_e status;
    // ip white list
    list_string_node_t ip_white_list;
} accelerate_configuration_t;

/**
 * list_cors_rule_node_t
 */
typedef struct list_cors_rule_node_t{
    struct list_head node;
    oos_string_t id;
    list_string_node_t allowed_origin_list;
    list_string_node_t allowed_method_list;
    list_string_node_t allowed_header_list;
    list_string_node_t expose_header_list;
    unsigned int  maxage_seconds;
} list_cors_rule_node_t;

typedef struct cors_configuration_t{
    list_cors_rule_node_t cors_rule_list;
}cors_configuration_t;

/**
 *  bucket的生命周期
 */
typedef struct lifecycle_configuration_t {
    struct list_head node;
    //规则的唯一标识。可以为空串，系统会自动分配一个ruleid
    oos_string_t ruleid;
    //要使用此规则的对象前缀。可以为空串，表示不限制前缀
    oos_string_t prefix;
    //Enabled：规则立即生效，Disabled：规则不生效
    oos_string_t status;
    //天数。正整数，对象更新时间加上此天数的时间被认为到期时间，days和date二选一，不选此项填0
    int days;
    //UTC零点日期。对象更新时间早于此时间则被认为过期，days和date二选一，不选此项填空串    
    oos_string_t date;
}lifecycle_configuration_t;

/**
*  bucket website config
*/
typedef struct website_configuration_t {
    // 在请求websiteendpoint 上的路径时，Suffix 会被加在请求的后面。
    // 例如，如果 suffix 是 Index.html，而你请求的是 bucket/images/，那么返回的响 应是名为 images/index.html 的 object
    oos_string_t suffix;
    //如果出现 4XX 错误，会返回指定 的 Object
    oos_string_t key;
}website_configuration_t;

/**
*  bucket logging config
*/
typedef struct logging_configuration_t
{
    // 选填配置，指定要保存 log 的 bucket，OOS 会向此 bucket 存储日志。可以设置任意一个你拥有的 bucket 作为 TargetBucket，
    // 包括启动日志的 bucket 本身。你也可以设置将多个 bucket 的日志存放到一个 TargetBucket 中，
    //  在这种 情况下，你需要为每个源 bucket 设置不同 的 TargetPrefix，以便不同 bucket 的 log 可 以被区分出来。
    oos_string_t targetBucket;
    // 选填配置，生成的 log 文件将以此为前缀命名
    oos_string_t targetPrefix;
    // 选填配置，在异地互备过程中，目标资源池的 log 会保 存到此 bucket 中，oos 会向此 bucket 存储日志
    oos_string_t triggerTargetBucket;
    // 选填配置，在异地互备过程中，目标资源池的 log 文件 将以此为前缀命名
    oos_string_t triggerTargetPrefix;
    // 选填配置，在异地互备过程中，源资源池的 log 会保存 到此 bucket 中，oos 会向此 bucket 存储日志
    oos_string_t triggerSourceBucket;
    // 选填配置，在异地互备过程中，源资源池的 log 文件将 以此为前缀命名
    oos_string_t triggerSourcePrefix;
}logging_configuration_t;


typedef struct trigger_configuration_t
{
    // 必填配置，Trigger 的名称，是字母或数字，不能包含特 殊符号，最多 20 个字符。
    oos_string_t triggerName;
    //必填配置，是否是默认的 trigger.枚举值为 ： true 、 false
    oos_string_t isDefault;
    // 选填配置，异地资源池的 endpoint
    oos_string_t remontEndPoint;
    //选填配置，同步到异地资源池的副本模式，如果不填写，会使用动态副本模式
    oos_string_t replicaMode;
    //如果填写了RemontEndPoint，则此值必填。 异地资源池的 bucketName
    oos_string_t remoteBucketName;
    //如果填写了RemontEndPoint，则此值必填。异地资源池的 AccessKey
    oos_string_t remoteAK;
    //如果填写了RemontEndPoint，则此值必填。异地资源池的 secretKey
    oos_string_t remoteSK;
    struct list_head node;
}trigger_configuration_t;


typedef struct oos_list_object_t{
    //content 头指针
    struct list_head* root;
    //common prefix 头指针
    struct list_head* prefix_root;
    //Bucket 的名称
    oos_string_t name; 
    //关键字以特定的前缀开始
    oos_string_t prefix;
    //标记从哪个位置开始罗列出bucket中的object
    oos_string_t marker;
	oos_string_t nextMarker;
    oos_string_t maxKeys;
    //将prefix和第一次出现delimiter的所有查询结果滚动的存入CommonPrefix组中。
    oos_string_t delimiter;
    //通过是（True）或否（false）来表示返回的结果是否为所有要求的结果
    oos_string_t isTruncated;
}oos_list_object_t;

typedef struct oos_list_object_content_t{
    struct list_head node;
    //对象的关键字
    oos_string_t key;
    //记录的对象最后一个被修改的日期和时间
    oos_string_t lastModified;
    //通过MD5的方式计算出标签，ETag主要用来反映对象内容的信息发生了改变，并不反映元数据的变化
    oos_string_t eTag;
    //记录对象object的大小
    oos_string_t size;
    //STANDARD或REDUCED_REDUNDANCY或EC_N_M
    oos_string_t storageClass;
    oos_string_t owner_id;
    //Object的所有者显示名称
    oos_string_t owner_DisplayName;
}oos_list_object_content_t;

typedef struct oos_list_object_common_prefix_t{
    struct list_head node;
    // 当定义delimiter之后，返回结果中会包含CommonPrefixes，CommonPrefix中包含以prefix开头，
    // delimiter结束的左右字符串组合，比如prefix是note/,同时delimiter是斜杠（/），
    //结果中的note/summer/ju将返回note/summer/,其余结果将按照maxkey要求返回。
    oos_string_t prefix;
}oos_list_object_common_prefix_t;

typedef struct oos_list_bucket_t{
    struct list_head* root;
    //Bucket的拥有者的信息
    oos_string_t owner_id;
    //Bucket 拥有者的用户显示姓名
    oos_string_t owner_DisplayName;
}oos_list_bucket_t;

typedef struct oos_list_bucket_content_t{
    struct list_head node;
    //Bucket 的名称
    oos_string_t name;
    //Bucket的创建日期
    oos_string_t creationDate;
}oos_list_bucket_content_t;

typedef struct oos_list_access_key_t{
    struct list_head* root;
    //用户名称
    oos_string_t userName;
    //通过是（True）或否（false）来表示返回的结果是否为所有要求的结果
    oos_string_t isTruncated;
    //标记从哪个位置开始罗列出access
    oos_string_t marker;
}oos_list_access_key_t;

typedef struct oos_list_access_key_member_t{
    struct list_head node;
    //access key 所属用户名称
    oos_string_t userName;
    oos_string_t accessKeyId;
    //默认是Active状态  InActive
    oos_string_t status;
    //是否是主key
    oos_string_t isPrimary;
    oos_string_t secretAccessKey;
}oos_list_access_key_member_t;

typedef struct initiate_multipart_upload_result_t{
    //分片上传对应的Bucket的名称
    oos_string_t bucket;
    //分片上传对应的对象名称
    oos_string_t key;
    //分片上传ID
    oos_string_t uploadId;
}initiate_multipart_upload_result_t;

typedef struct oos_list_multipart_uploads_t{
    //分片上传对应的对象名称
    oos_string_t bucket;  
    //指定key值，在这个key当前位置或它之后开始列表操作
    oos_string_t keyMarker;  
    //指定分片上传ID，在这个ID所在位置之后开始列表操作
    oos_string_t uploadIdMarker; 
    //当此次列表不能将所有正在执行的分片上传过程列举完成时，NextKeyMarker作为下一次列表请求的key-marker参数的值。 
    oos_string_t nextKeyMarker;  
    //当此次列表不能将所有正在执行的分片上传过程列举完成时，NextKeyMarker作为下一次列表请求的upload-id-marker参数的值。
    oos_string_t nextUploadIdMarker;  
    //响应中包含的上传过程的最大数目
    oos_string_t maxUploads;  
    //标识此次分片上传过程中的所有片段是否全部被列出，如果为true则表示没有全部列出。
    //如果分片上传过程的片段数超过了MaxParts元素指定的最大数，则会导致一次列表请求无法将所有片段数列出
    oos_string_t isTruncated;  

    struct list_head* root;
	//common prefix 头指针
	struct list_head* prefix_root;
}oos_list_multipart_uploads_t;

typedef struct oos_list_multipart_uploads_content_t{
    //对象的Key
    oos_string_t key;
    //分片上传ID
    oos_string_t uploadId;
    //指定执行此次分片上传过程的用户账号。OOS账号的ID号
    oos_string_t initiator_id;
    //OOS账号的账户名
    oos_string_t initiator_displayName;
    //用来标识对象的拥有者id
    oos_string_t owner_id;
    //用来标识对象的拥有者显示名称
    oos_string_t owner_displayName;
    oos_string_t storageClass;
    //指定执行此次分片上传过程的用户账号
    oos_string_t initiated;
    struct list_head node;
}oos_list_multipart_uploads_content_t;

typedef struct oos_list_multipart_uploads_common_prefix_t {
	struct list_head node;
	// 当定义delimiter之后，返回结果中会包含CommonPrefixes，CommonPrefix中包含以prefix开头，
	// delimiter结束的左右字符串组合，比如prefix是note/,同时delimiter是斜杠（/），
	//结果中的note/summer/ju将返回note/summer/,其余结果将按照maxkey要求返回。
	oos_string_t prefix;
}oos_list_multipart_uploads_common_prefix_t;

/**
 *  片段列表
 *  @param partnumber  标识片段的分片号
 *  @param etag  片段上传完成时返回的Etag值
 */
typedef struct part_etags {
    //分片号
    int partnumber;
    //片段上传完成时返回的Etag内容。
    oos_string_t etag;
}part_etags;

typedef struct part_listing_t{
    int maxParts;
    oos_string_t bucket;
    oos_string_t key;
    //用来标识一个分片上传过程
    oos_string_t uploadId;
    //指定执行此次分片上传过程的用户账号ID
    oos_string_t initiator_id;
    //指定执行此次分片上传过程的用户账号显示名
    oos_string_t initiator_displayName;
    //OOS账号的ID号
    oos_string_t owner_id;
    //OOS账号的账户名
    oos_string_t owner_displayName;
    //对象的存储类型（STANDARD或REDUCED_REDUNDANCY或EC_N_M）
    oos_string_t storageClass;
    //标识此次分片上传过程中的所有片段是否全部被列出，如果为true则表示没有全部列出。
    //如果分片上传过程的片段数超过了MaxParts元素指定的最大数，则会导致一次列表请求无法将所有片段数列出
    oos_string_t isTruncated;
    struct list_head* root;
}part_listing_t;

typedef struct part_summary_t{
    //标识片段的分片号
    int partNumber;
    //片段的数据大小
    int size;
    //片段上传完成的日期
    oos_string_t lastModified;
    //片段上传完成时返回的ETag值
    oos_string_t eTag;
    struct list_head node;
}part_summary_t;

typedef struct copy_object_result_t{
    oos_string_t etag;
    oos_string_t lastModifiedDate;
}copy_object_result_t;

typedef enum bucket_type_e{BUCKET_TYPE_PRIVATE = 0, BUCKET_TYPE_PUBLIC_READ, BUCKET_TYPE_PUBLIC} bucket_type_e;
static inline void add_list_string_node_pool(oos_pool_t *p, const char* value, list_string_node_t* list){
    list_string_node_t * string_node;
    string_node = oos_pcalloc(p, sizeof(*string_node));
    INIT_LIST_HEAD(&string_node->node);
    oos_str_set(p, &string_node->text,value);
    list_add_tail(&string_node->node, &list->node);
}

static inline void add_list_object_node_pool(oos_pool_t *p, const char* value, list_delete_node_t* list){
    list_delete_node_t * object_node;
    object_node = oos_pcalloc(p, sizeof(*object_node));
    INIT_LIST_HEAD(&object_node->node);
    oos_str_set(p, &object_node->object_key,value);
    list_add_tail(&object_node->node, &list->node);
}



static inline void add_list_string_node(request_options_t *options, const char* value, list_string_node_t* list){
  add_list_string_node_pool(options->pool,  value, list);
}



#endif
