
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "curl/curl.h"
#include "oos_api.h"
#include "make_header.h"
#include "response.h"
#include "xml_util.h"
#include "file_deal.h"
#include "oos_status.h"
#include "util.h"

#include <inttypes.h>


static oos_status_t *check_params(const oos_string_t* parameter_value, oos_status_t *status, const char* error_msg) {
    if(oos_string_is_empty(parameter_value))
    {
        status->code = STATUS_INVALID_PARAMETER;
        status->error_msg = error_msg;
    }
    return status;
}

void config_status(const int code,const char* error_msg, oos_status_t *status){
    status->code = code;
    status->error_msg = error_msg;
}

static int check_buf(const char * buf_data, int buf_len) {
    if (NULL == buf_data || buf_len <= 0) {
        fprintf(stderr, "buf_data is NULL or buf_len less than zero\n");
        return -1;
    }
    return 0;
}

oos_status_t *parse_status_from_response(buffer_t* resp, int* err, oos_status_t *status)
{
    printf("&&response status code: %d\n",resp-> status_code);
    printf("&&response body=%s\n", resp-> body);
    printf("&&response content=%s\n", resp-> content);
    printf("&&response header: %s\n",resp-> header);
    printf("&&response status_msg: %s\n", resp-> status_msg);
    printf("&&response error_msg: %s\n",resp-> error_msg);
    printf("&&response etag: %s\n",resp-> etag);
    printf("&& err %d\n", *err);
    if(*err != 0)
    {
        status->code = 0 - *err;
        status->error_msg = curl_easy_strerror(*err);
        return status;
    }
    if(resp->status_code >= 300)
    {
        status->code = resp->status_code;
        status->error_msg = resp->error_msg;
    }

    return status;
}
/**

*/
//---------------------以下是接口实现-------------------------
oos_status_t *oos_list_buckets(const request_options_t* options, oos_list_bucket_t **buckets,
                                oos_table_t** resp_header) {
    oos_status_t *status = oos_status_create(options->pool);
    buffer_t *resp = buffer_init(options->pool);
    int err = 0;
    make_header(options, GET_METHOD, NULL, NULL, NULL, NULL, NULL, resp, &err);
    parse_status_from_response(resp, &err, status);
    *resp_header = oos_table_clone(options->pool, resp->headers);
    if(oos_status_is_ok(status) && resp->body)
    {
        *buckets = oos_list_bucket_parse_from_body(options->pool,resp-> body);
    }

    buffer_free(resp);
    return status;
}

oos_status_t* oos_list_objects(const request_options_t* options,
                         const char* bucket, const char* prefix,const char* delimiter, 
                         const char* marker, const char* max_key, oos_list_object_t **objects, oos_table_t** resp_header) {
    oos_status_t *status = oos_status_create(options->pool);
    if (!bucket || strlen(bucket)<= 0) {
        config_status(STATUS_INVALID_PARAMETER,
            "bucket need not null\n",status);
        return status;
    }
    buffer_t *resp = buffer_init(options->pool);
    int err = 0;
    char query[1024] = { '\0' };
    char internal_query[1024] = { '\0' };
    if (prefix && strlen(prefix) > 0) {
		if(strlen(internal_query)>0){
			strcat(internal_query, "&");
		}
        snprintf(query, 1024, "prefix=%s", prefix);
        strcat(internal_query, query);
    }
   if (delimiter && strlen(delimiter) > 0) {
	   if (strlen(internal_query)>0) {
		   strcat(internal_query, "&");
	   }
       snprintf(query, 1024, "delimiter=%s", delimiter);
       strcat(internal_query, query);
   }
   if (marker && strlen(marker) > 0) {
	   if (strlen(internal_query)>0) {
		   strcat(internal_query, "&");
	   }
       snprintf(query, 1024, "marker=%s", marker);
       strcat(internal_query, query);
   } 
   if (max_key && strlen(max_key) > 0) {
	   if (strlen(internal_query)>0) {
		   strcat(internal_query, "&");
	   }
	   snprintf(query, 1024, "max-keys=%s", max_key);
	   strcat(internal_query, query);
   }
    
    make_header(options, GET_METHOD, bucket, NULL, NULL, internal_query, NULL,
                resp, &err);

    parse_status_from_response(resp, &err, status);
    *resp_header = oos_table_clone(options->pool, resp->headers);
    if(NULL != resp->body && resp->status_code < 300){
        *objects =  oos_list_objects_parse_from_body(options->pool, resp->body);
    }
    buffer_free(resp);
    return status;
}

oos_status_t* oos_create_bucket(const request_options_t* options,
                   const char* bucket, bucket_type_e type, struct location_configuration_t* config, oos_table_t** resp_header) {
    oos_status_t *status = oos_status_create(options->pool);
    if (!bucket || strlen(bucket)<= 0) {
        config_status(STATUS_INVALID_PARAMETER,
            "bucket need not null\n",status);
        return status;
    }

    if (!config ) {
        config_status(STATUS_INVALID_PARAMETER,
            "config need not null\n",status);
        return status;
    }

    buffer_t *resp = buffer_init(options->pool);
    int err = 0;

    char bucket_acl[2048] = {NULL};
    switch (type) {
        case BUCKET_TYPE_PUBLIC:
        sprintf(bucket_acl, "%s: %s", "x-amz-acl", "public-read-write");
        break;
    case BUCKET_TYPE_PUBLIC_READ:
        sprintf(bucket_acl, "%s: %s", "x-amz-acl", "public-read");
        break;
    case BUCKET_TYPE_PRIVATE:
        sprintf(bucket_acl, "%s: %s", "x-amz-acl", "private");
        break;
    default:
        printf("unknown bucket type !!!");
        break;
    }

    long buf_size = 5 * 1024 * 1024;
    char* buf = (char *)oos_pcalloc(options->pool,buf_size);
    int dat_len = 0;
    int dat_size = buf_size;
    int print_len = 0;
    char *dat = buf;

    if(config){
        print_len = snprintf(dat + dat_len, dat_size - dat_len, "<CreateBucketConfiguration><MetadataLocationConstraint><Location>%s</Location></MetadataLocationConstraint>",
                             config->medadata_location.data);
        dat_len += print_len;

        char data_location_type[64] = {"\0"};
        if(LOCAL == config->data_location_type){
            strcpy(data_location_type, "Local");
        }else{
            strcpy(data_location_type, "Specified");
        }

        print_len = snprintf(dat + dat_len, dat_size - dat_len, "<DataLocationConstraint><Type>%s</Type><LocationList>",
                             data_location_type);
        dat_len += print_len;

        list_string_node_t* tmp;
        list_for_each_entry(tmp, &config->data_location_list.node, node)
        {
            print_len = snprintf(dat + dat_len, dat_size - dat_len, "<Location>%s</Location>",
                                 tmp->text.data);
            dat_len += print_len;
        }

        char data_location_ScheduleStrategy[64] = {"\0"};
        if(ALLOWED == config->data_schedule_strategy){
            strcpy(data_location_ScheduleStrategy, "Allowed");
        }else{
            strcpy(data_location_ScheduleStrategy, "NotAllowed");
        }

        print_len = snprintf(dat + dat_len, dat_size - dat_len, "</LocationList><ScheduleStrategy>%s</ScheduleStrategy></DataLocationConstraint></CreateBucketConfiguration>",
                             data_location_ScheduleStrategy);
        dat_len += print_len;
        printf("location is:%s\n",dat);
    }

    make_header_buf(options, PUT_METHOD, bucket, NULL, dat,dat_len , NULL, bucket_acl,
                resp, &err);
    parse_status_from_response(resp, &err, status);
    *resp_header = oos_table_clone(options->pool, resp->headers);
    buffer_free(resp);
    return status;
}

oos_status_t* oos_head_bucket(const request_options_t* options,
                 const char* bucket, oos_table_t** resp_header) {
    oos_status_t *status = oos_status_create(options->pool);
    if (!bucket || strlen(bucket)<= 0) {
        config_status(STATUS_INVALID_PARAMETER,
            "bucket need not null\n",status);
        return status;
    }

    buffer_t *resp = buffer_init(options->pool);
    int err = 0;
    make_header(options, HEAD_METHOD, bucket, NULL, NULL,
                NULL, NULL, resp, &err);

    parse_status_from_response(resp, &err, status);
    *resp_header = oos_table_clone(options->pool, resp->headers);
    buffer_free(resp);
    return status;
}

oos_status_t* oos_delete_bucket(const request_options_t* options,
                   const char* bucket, oos_table_t** resp_header) {
    oos_status_t *status = oos_status_create(options->pool);

    if (!bucket || strlen(bucket)<= 0) {
        config_status(STATUS_INVALID_PARAMETER,
            "bucket need not null\n",status);
        return status;
    }
    buffer_t *resp = buffer_init(options->pool);

    int err = 0;
    make_header(options, DELETE_METHOD, bucket, NULL, NULL, NULL, NULL,
                resp, &err);
    parse_status_from_response(resp, &err, status);
    *resp_header = oos_table_clone(options->pool, resp->headers);
    buffer_free(resp);
    return status;
}

oos_status_t* oos_put_object_from_buffer(const request_options_t* options,
                    const char* bucket, const char* object_key,
                    const char* buf_data, int buf_len,
                    oos_table_t* req_header, oos_table_t** resp_header) {
   oos_status_t *status = oos_status_create(options->pool);
   buffer_t *resp = buffer_init(options->pool);

    int err = 0;
    if (check_buf(buf_data, buf_len) != 0) {
        config_status(STATUS_INVALID_PARAMETER,"buf_data is NULL or buf_len less than zero\n", status);

        return status;
    }
    if (!bucket || strlen(bucket)<= 0) {
        config_status(STATUS_INVALID_PARAMETER,
            "bucket need not null\n",status);
        return status;
    }
    if (!object_key || strlen(object_key)<= 0) {
        config_status(STATUS_INVALID_PARAMETER,
            "object_key need not null\n",status);
        return status;
    }
    /*
    char object_folder[1024] = {'\0'};
    if(folder)
    {
        strcpy(object_folder,folder);
        strcat(object_folder,"/");
        strcat(object_folder,objectname);
    }
    else
        strcpy(object_folder,objectname);
	*/

    int pos = 0;
    char *headers = NULL;
    oos_array_header_t *tarr;
    oos_table_entry_t *telts;

    if (req_header) {
        tarr = oos_table_elts(req_header);
        telts = (oos_table_entry_t*)tarr->elts;
        for (pos = 0; pos < tarr->nelts; ++pos) {
            if (NULL == headers){
                headers = apr_psprintf(options->pool, "%s: %s",telts[pos].key, telts[pos].val);
            }
            else{
                headers = apr_psprintf(options->pool, "%s\n%s: %s", headers, telts[pos].key, telts[pos].val);
            }
        }
    }
    
    make_header_buf(options, PUT_METHOD, bucket, object_key, buf_data,
                    buf_len, NULL, headers, resp, &err);
    parse_status_from_response(resp, &err, status);
    *resp_header = oos_table_clone(options->pool, resp->headers);

    buffer_free(resp);
    return status;
}

oos_status_t* oos_object_generate_presigned_url(const request_options_t* options,
                    const char* bucket, const char* object_key,
                    const int expires_sec,
                    oos_table_t** resp_header, char** pre_signed_url) {
   oos_status_t *status = oos_status_create(options->pool);
   buffer_t *resp = buffer_init(options->pool);

    int err = 0;
    if (!bucket || strlen(bucket)<= 0) {
        config_status(STATUS_INVALID_PARAMETER,
            "bucket need not null\n",status);
        return status;
    }
    if (!object_key || strlen(object_key)<= 0) {
        config_status(STATUS_INVALID_PARAMETER,
            "object_key need not null\n",status);
        return status;
    }

    int pos = 0;
    char *headers = NULL;
    oos_array_header_t *tarr;
    oos_table_entry_t *telts;

    make_header_presigned_url(options, GET_METHOD, bucket, object_key, NULL,
                    0, expires_sec, NULL, headers, resp, pre_signed_url);
    parse_status_from_response(resp, &err, status);
    *resp_header = oos_table_clone(options->pool, resp->headers);

    buffer_free(resp);
    return status;
}


oos_status_t* oos_put_object_from_file(const request_options_t* options,
                     const char* bucket, const char* object_key, const char* file_path,
                     oos_table_t* req_header, oos_table_t** resp_header) {
    oos_status_t *status = oos_status_create(options->pool);
    if (!bucket || strlen(bucket)<= 0) {
        config_status(STATUS_INVALID_PARAMETER,
            "bucket need not null\n",status);
        return status;
    }
    if (!object_key || strlen(object_key)<= 0) {
        config_status(STATUS_INVALID_PARAMETER,
            "object_key need not null\n",status);
        return status;
    }
    buffer_t *resp = buffer_init(options->pool);
    int err = 0;

    int pos = 0;
    char *headers = NULL;
    oos_array_header_t *tarr;
    oos_table_entry_t *telts;

    if (req_header) {
        tarr = oos_table_elts(req_header);
        telts = (oos_table_entry_t*)tarr->elts;
        for (pos = 0; pos < tarr->nelts; ++pos) {
            if (NULL == headers){
                headers = apr_psprintf(options->pool, "%s: %s",telts[pos].key, telts[pos].val);
            }
            else{
                headers = apr_psprintf(options->pool, "%s\n%s: %s", headers, telts[pos].key, telts[pos].val);
            }
        }
    }
    make_header_file(options, PUT_METHOD, bucket, object_key, file_path,
                     "", headers, resp, &err);
    parse_status_from_response(resp, &err, status);
    *resp_header = oos_table_clone(options->pool, resp->headers);
    buffer_free(resp);
    return status;
}

/**
    get object to buf md5计算出错
*/
oos_status_t* oos_get_object_to_buffer(const request_options_t* options, const char* bucket,
                    const char* object_key, oos_string_t* buf,
	oos_table_t** resp_meta, oos_table_t** resp_header) {
	return oos_get_object_to_buffer_ex(options, bucket, object_key, NULL, buf, resp_meta, resp_header);
}
oos_status_t* oos_get_object_to_buffer_ex(const request_options_t* options, const char* bucket,
	                    const char* object_key,  oos_table_t* req_header, oos_string_t* buf,
                     oos_table_t** resp_meta, oos_table_t** resp_header) {
    oos_status_t *status = oos_status_create(options->pool);
    if (!bucket || strlen(bucket)<= 0) {
        config_status(STATUS_INVALID_PARAMETER,
            "bucket need not null\n",status);
        return status;
    }
    if (!object_key || strlen(object_key)<= 0) {
        config_status(STATUS_INVALID_PARAMETER,
            "object_key need not null\n",status);
        return status;
    }
    int pos = 0;
    char *headers = NULL;
    oos_array_header_t *tarr;
    oos_table_entry_t *telts;
    int is_range = 0;
    if (req_header) {
        tarr = oos_table_elts(req_header);
        telts = (oos_table_entry_t*)tarr->elts;
        for (pos = 0; pos < tarr->nelts; ++pos) {
            if(strcmp(telts[pos].key, "Range")==0){
               is_range = 1;
            }
            if (NULL == headers){
                headers = apr_psprintf(options->pool, "%s: %s",telts[pos].key, telts[pos].val);
            }
            else{
                headers = apr_psprintf(options->pool, "%s\n%s: %s", headers, telts[pos].key, telts[pos].val);
            }
        }
    }
    buffer_t *resp = buffer_init(options->pool);

    int err = 0;
    make_header_buf(options, GET_METHOD, bucket, object_key, NULL,
                    -1, NULL, headers, resp, &err);
    parse_status_from_response(resp, &err, status);
    *resp_header = oos_table_clone(options->pool, resp->headers);
    parse_meta_data(options->pool, resp->header, resp_meta);
    if(NULL != resp->content && resp->status_code < 300&&resp->content_length > 0){
	buf->data = (char*)oos_pcalloc(options->pool, resp->content_length);
        memcpy(buf->data, resp->content, resp->content_length);
        buf->len = resp->content_length;
    }
    //分片上传的对象都没有etag。 所以但凡etag值为空的都不进行校验。 
    if(NULL == resp->etag || strcmp(resp->etag,"") == 0){
        buffer_free(resp);
        return status;
    }

    char base64_buf[128];
    compute_buf_md5(buf->data, buf->len, base64_buf);
    if(0 == is_range && strcmp(resp->etag,base64_buf) != 0){
        config_status(STATUS_INVALID_PARAMETER,
            "No MD5 digest algorithm available.  Unable to calculate checksum and verify data integrity.",status);
    }

    buffer_free(resp);
    return status;
}

oos_status_t* oos_get_object_to_file(const request_options_t* options,
                     const char* bucket, const char* object_key, const char* savepath,
	oos_table_t** resp_meta, oos_table_t** resp_header) {
	return oos_get_object_to_file_ex(options, bucket, object_key, NULL, savepath, resp_meta, resp_header);
}
oos_status_t* oos_get_object_to_file_ex(const request_options_t* options,
                     const char* bucket, const char* object_key, oos_table_t* req_header, const char* savepath,
                     oos_table_t** resp_meta, oos_table_t** resp_header) {
    oos_status_t *status = oos_status_create(options->pool);
    if (!bucket || strlen(bucket)<= 0) {
        config_status(STATUS_INVALID_PARAMETER,
            "bucket need not null\n",status);
        return status;
    }
    if (!object_key || strlen(object_key)<= 0) {
        config_status(STATUS_INVALID_PARAMETER,
            "object_key need not null\n",status);
        return status;
    }
	int pos = 0;
	char *headers = NULL;
	oos_array_header_t *tarr;
	oos_table_entry_t *telts;
        int is_range = 0;
	if (req_header) {
		tarr = oos_table_elts(req_header);
		telts = (oos_table_entry_t*)tarr->elts;
		for (pos = 0; pos < tarr->nelts; ++pos) {
                        if(strcmp(telts[pos].key, "Range")==0){
                            is_range = 1;
                        }
			if (NULL == headers) {
				headers = apr_psprintf(options->pool, "%s: %s", telts[pos].key, telts[pos].val);
			}
			else {
				headers = apr_psprintf(options->pool, "%s\n%s: %s", headers, telts[pos].key, telts[pos].val);
			}
		}
	}
    buffer_t *resp = buffer_init(options->pool);
    int err = 0;
    
    make_header_file(options, GET_METHOD, bucket, object_key, savepath,
                     NULL, headers, resp, &err);
    parse_status_from_response(resp, &err, status);
    *resp_header = oos_table_clone(options->pool, resp->headers);
    parse_meta_data(options->pool, resp->header, resp_meta);
    //分片上传的对象都没有etag。 所以但凡etag值为空的都不进行校验。 
    if(NULL == resp->etag || strcmp(resp->etag,"") == 0){
        buffer_free(resp);
        return status;
    }

    char base64_buf[128];
    compute_file_md5(savepath, base64_buf);

    if(0 == is_range && strcmp(resp->etag,base64_buf) != 0){
        config_status(STATUS_INVALID_PARAMETER,
            "No MD5 digest algorithm available.  Unable to calculate checksum and verify data integrity.",status);
    }
    buffer_free(resp);
    return status;
}

oos_status_t* oos_head_object(const request_options_t* options,
                 const char* bucket, const char* object_key, oos_table_t* req_header, oos_table_t** resp_header) {
    oos_status_t *status = oos_status_create(options->pool);
    if (!bucket || strlen(bucket)<= 0) {
        config_status(STATUS_INVALID_PARAMETER,
            "bucket need not null\n",status);
        return status;
    }
    if (!object_key || strlen(object_key)<= 0) {
        config_status(STATUS_INVALID_PARAMETER,
            "object_key need not null\n",status);
        return status;
    }
	int pos = 0;
	char *headers = NULL;
	oos_array_header_t *tarr;
	oos_table_entry_t *telts;
	int is_range = 0;
	if (req_header) {
		tarr = oos_table_elts(req_header);
		telts = (oos_table_entry_t*)tarr->elts;
		for (pos = 0; pos < tarr->nelts; ++pos) {
			if (strcmp(telts[pos].key, "Range") == 0) {
				is_range = 1;
			}
			if (NULL == headers) {
				headers = apr_psprintf(options->pool, "%s: %s", telts[pos].key, telts[pos].val);
			}
			else {
				headers = apr_psprintf(options->pool, "%s\n%s: %s", headers, telts[pos].key, telts[pos].val);
			}
		}
	}
    buffer_t *resp = buffer_init(options->pool);
    int err = 0;
    make_header(options, HEAD_METHOD, bucket, object_key, NULL,
                NULL, headers, resp, &err);

    parse_status_from_response(resp, &err, status);
    *resp_header = oos_table_clone(options->pool, resp->headers);
    buffer_free(resp);
    return status;
}

oos_status_t* oos_delete_object(const request_options_t* options,
                   const char* bucket, const char* object_key, oos_table_t** resp_header) {
    oos_status_t *status = oos_status_create(options->pool);
    if (!bucket || strlen(bucket)<= 0) {
        config_status(STATUS_INVALID_PARAMETER,
            "bucket need not null\n",status);
        return status;
    }
    if (!object_key || strlen(object_key)<= 0) {
        config_status(STATUS_INVALID_PARAMETER,
            "object_key need not null\n",status);
        return status;
    }
    buffer_t *resp = buffer_init(options->pool);

    int err = 0;
    make_header(options, DELETE_METHOD, bucket, object_key, NULL,
                NULL, NULL, resp, &err);
    parse_status_from_response(resp, &err, status);
    *resp_header = oos_table_clone(options->pool, resp->headers);
    buffer_free(resp);
    return status;
}

oos_status_t* oos_delete_multiple_object(const request_options_t* options,
                   const char* bucket, delete_multiple_list_t* object_list, oos_bool is_quiet, oos_table_t** resp_header) {
    oos_status_t *status = oos_status_create(options->pool);
    if (!bucket || strlen(bucket)<= 0) {
        config_status(STATUS_INVALID_PARAMETER,
            "bucket need not null\n",status);
        return status;
    }
    if (!object_list ) {
        config_status(STATUS_INVALID_PARAMETER,
            "object_key need not null\n",status);
        return status;
    }

    long buf_size = 5 * 1024 * 1024;
    char* buf = (char *)oos_pcalloc(options->pool,buf_size);
    int dat_len = 0;
    int dat_size = buf_size;
    int print_len = 0;
    char *dat = buf;


    print_len = snprintf(dat + dat_len, dat_size - dat_len, "<Delete>");
    dat_len += print_len;

	// Quiet
	char s_quiet[16] = { "\0" };
	if (is_quiet == OOS_TRUE) {
		strcpy(s_quiet, "true");
	}
	else {
		strcpy(s_quiet, "false");
	}
	print_len = snprintf(dat + dat_len, dat_size - dat_len, "<Quiet>%s</Quiet>", s_quiet);
	dat_len += print_len;

    // object list
    list_string_node_t* tmp;
    int count = 0;
    list_for_each_entry(tmp, &object_list->delete_list.node, node)
    {
        print_len = snprintf(dat + dat_len, dat_size - dat_len, "<Object><Key>%s</Key></Object>",
                             tmp->text.data);
        dat_len += print_len;
        count++;
    }
    if(0 == count){
        config_status(STATUS_INVALID_PARAMETER,
            "object_key need not null\n",status);
        return status;
    }

    print_len = snprintf(dat + dat_len, dat_size - dat_len, "</Delete>");
    dat_len += print_len;

    printf("delete list is:%s\n",dat);

    buffer_t *resp = buffer_init(options->pool);

    int err = 0;

    char internal_query[16] = { '\0' };
    snprintf(internal_query, 16, "delete");

    char header_str[1024];
    char  base64_buf[32];
    compute_buf_md5b64(dat, dat_len, base64_buf);
    snprintf(header_str, 128, "Content-Md5: %s", base64_buf);

    make_lifecycle(options, POST_METHOD, bucket, NULL,
                   dat, dat_len, internal_query, header_str,
                   resp, &err);

    if(NULL != resp->body && resp->status_code < 300 && is_quiet != OOS_TRUE){
        oos_delete_result_parse_from_body(options->pool, resp->body, object_list);
    }
	if (is_quiet == OOS_TRUE) {
		list_del_init(&object_list->delete_list.node);
	}

    parse_status_from_response(resp, &err, status);
    *resp_header = oos_table_clone(options->pool, resp->headers);
    buffer_free(resp);
    return status;
}

/**
*/
oos_status_t* oos_copy_object(const request_options_t* options, const char* src_bucket,
                 const char* src_objectname, const char* dst_bucket,
                 const char* dst_objectname,copy_object_result_t** result,
                 oos_table_t* req_header, oos_table_t** resp_header) {
    oos_status_t *status = oos_status_create(options->pool);
    buffer_t *resp = buffer_init(options->pool);

    if (!src_bucket || strlen(src_bucket)<= 0) {
        config_status(STATUS_INVALID_PARAMETER,
            "source bucket need not null\n",status);
        return status;
    }
    if (!src_objectname || strlen(src_objectname)<= 0) {
        config_status(STATUS_INVALID_PARAMETER,
            "source object need not null\n",status);
        return status;
    }
    if (!dst_objectname || strlen(dst_objectname)<= 0) {
        config_status(STATUS_INVALID_PARAMETER,
            "target object need not null\n",status);
        return status;
    }
    if (!dst_bucket || strlen(dst_bucket)<= 0) {
        config_status(STATUS_INVALID_PARAMETER,
            "target bucket need not null\n",status);
        return status;
    }

    int err = 0;
    int pos = 0;
    char *headers = NULL;
    oos_array_header_t *tarr;
    oos_table_entry_t *telts;

    if (req_header) {
        tarr = oos_table_elts(req_header);
        telts = (oos_table_entry_t*)tarr->elts;
        for (pos = 0; pos < tarr->nelts; ++pos) {
            if (NULL == headers){
                headers = apr_psprintf(options->pool, "%s: %s",telts[pos].key, telts[pos].val);
            }
            else{
                headers = apr_psprintf(options->pool, "%s\n%s: %s", headers, telts[pos].key, telts[pos].val);
            }
        }
    }

    if(src_bucket != NULL && src_objectname != NULL){
        char *encode_src_bucket = src_bucket, *encode_src_objectname = src_objectname;
        encode_src_bucket = oos_palloc(options->pool, strlen(src_bucket)*3);
        urlEncode(encode_src_bucket, src_bucket, strlen(src_bucket));

        encode_src_objectname = oos_palloc(options->pool, strlen(src_objectname)*3);
        urlEncode(encode_src_objectname, src_objectname, strlen(src_objectname));
        headers = apr_psprintf(options->pool, "%s\nx-amz-copy-source: /%s/%s", headers, encode_src_bucket, encode_src_objectname);
    }

    make_header(options, PUT_METHOD, dst_bucket, dst_objectname,
                NULL, NULL, headers, resp, &err);
    if(NULL != resp->body && resp->status_code < 300){
        *result = oos_copy_object_parse_from_body(options->pool,resp->body); 
    }
    parse_status_from_response(resp, &err, status);
    *resp_header = oos_table_clone(options->pool, resp->headers);
    buffer_free(resp);  
    return status;
}


oos_status_t* oos_init_multipart_upload(const request_options_t* options,
                           const char* bucket, const char* object_key,
                           initiate_multipart_upload_result_t** result,oos_table_t* req_header,
                           oos_table_t** resp_header) {
    oos_status_t *status = oos_status_create(options->pool);
    if (!bucket || strlen(bucket)<= 0) {
        config_status(STATUS_INVALID_PARAMETER,
            "bucket need not null\n",status);
        return status;
    }
    if (!object_key || strlen(object_key)<= 0) {
        config_status(STATUS_INVALID_PARAMETER,
            "object_key need not null\n",status);
        return status;
    }
    buffer_t *resp = buffer_init(options->pool);

    int err = 0;
    char query_buf[16];
    snprintf(query_buf, 16, "uploads");
    int pos = 0;
    char *headers = NULL;
    oos_array_header_t *tarr;
    oos_table_entry_t *telts;

    if (req_header) {
        tarr = oos_table_elts(req_header);
        telts = (oos_table_entry_t*)tarr->elts;
        for (pos = 0; pos < tarr->nelts; ++pos) {
            if (NULL == headers){
                headers = apr_psprintf(options->pool, "%s: %s",telts[pos].key, telts[pos].val);
            }
            else{
                headers = apr_psprintf(options->pool, "%s\n%s: %s", headers, telts[pos].key, telts[pos].val);
            }
        }
    }

    make_multiparts(options, POST_METHOD, bucket, object_key, NULL, 0, query_buf,
                    headers, MULTI_OP, resp, &err);
    if(NULL != resp->body && resp->status_code < 300){
        *result = oos_multipart_uploadId_parse_from_body(options->pool,resp->body);
    }
    parse_status_from_response(resp, &err, status);
    *resp_header = oos_table_clone(options->pool, resp->headers);
    buffer_free(resp);
    return status;
}

oos_status_t* oos_upload_part_from_buffer(const request_options_t* options, const char* bucket,
                 const char* object_key, const char* upload_id,
                 int part_number, const char* part_data,int part_data_len, oos_string_t* etag,
                 oos_table_t** resp_header) {
    oos_status_t *status = oos_status_create(options->pool);
    if (!bucket || strlen(bucket)<= 0) {
        config_status(STATUS_INVALID_PARAMETER,
            "bucket need not null\n",status);
        return status;
    }

    if (!object_key || strlen(object_key)<= 0) {
        config_status(STATUS_INVALID_PARAMETER,
            "object_key need not null\n",status);
        return status;
    }

    int err = 0;
    if (check_buf(part_data, part_data_len) != 0) {
        config_status(STATUS_INVALID_PARAMETER,
            "buf_data is NULL or buf_len less than zero\n",status);
        return status;
    }
    if (upload_id == NULL || strlen(upload_id) <= 0) {
        config_status(STATUS_INVALID_PARAMETER,
            "upload id is null \n",status);
        return status;
    }
    if (part_number < 1 || part_number > 10000) {
        config_status(STATUS_INVALID_PARAMETER,
            "part number is 0 or more than 10000 \n",status);
        return status;
    }
    buffer_t *resp = buffer_init(options->pool);
    char internal_query[1024] = { '\0' };
    snprintf(internal_query, 1024, "partNumber=%d&uploadId=%s",
             part_number, upload_id);
    
    char header_str[1024];
    char  base64_buf[32];
    compute_buf_md5b64(part_data, part_data_len, base64_buf);
    snprintf(header_str, 1024, "Content-Md5: %s", base64_buf);
    
    make_header_buf(options, PUT_METHOD, bucket, object_key, part_data,
                    part_data_len, internal_query, header_str, resp, &err);
    if(resp->status_code < 300){
        oos_str_set(options->pool, etag,resp->etag);
    }
    parse_status_from_response(resp, &err, status);
    *resp_header = oos_table_clone(options->pool, resp->headers);
    buffer_free(resp);
    return status;
}

oos_status_t* oos_copy_part(const request_options_t* options, const char* bucket,
                 const char* object_key, const char* upload_id,
                 int part_number, const char* src_bucket,const char* src_object_key,
                 oos_table_t* req_header, oos_string_t* etag,
                 oos_table_t** resp_header) {
    oos_status_t *status = oos_status_create(options->pool);
    if (!bucket || strlen(bucket)<= 0) {
        config_status(STATUS_INVALID_PARAMETER,
            "bucket need not null\n",status);
        return status;
    }

    if (!object_key || strlen(object_key)<= 0) {
        config_status(STATUS_INVALID_PARAMETER,
            "object_key need not null\n",status);
        return status;
    }
    if (!src_bucket || strlen(src_bucket)<= 0) {
        config_status(STATUS_INVALID_PARAMETER,
            "src_bucket need not null\n",status);
        return status;
    }
    if (!src_object_key || strlen(src_object_key)<= 0) {
        config_status(STATUS_INVALID_PARAMETER,
            "src_object_key need not null\n",status);
        return status;
    }
    if (upload_id == NULL || strlen(upload_id) <= 0) {
        config_status(STATUS_INVALID_PARAMETER,
            "upload id is null \n",status);
        return status;
    }
    if (part_number < 1 || part_number > 10000) {
        config_status(STATUS_INVALID_PARAMETER,
            "part number is 0 or more than 10000 \n",status);
        return status;
    }

    int err = 0;
    int pos = 0;
    char *headers = NULL;

     // x-amz-copy-source
    headers = apr_psprintf(options->pool, "%s: /%s/%s", "X-Amz-Copy-Source", src_bucket, src_object_key);

    oos_array_header_t *tarr;
    oos_table_entry_t *telts;

    if (req_header) {
        tarr = oos_table_elts(req_header);
        telts = (oos_table_entry_t*)tarr->elts;
        for (pos = 0; pos < tarr->nelts; ++pos) {
            if (NULL == headers){
                headers = apr_psprintf(options->pool, "%s: %s",telts[pos].key, telts[pos].val);
            }
            else{
                headers = apr_psprintf(options->pool, "%s\n%s: %s", headers, telts[pos].key, telts[pos].val);
            }
        }
    }

    buffer_t *resp = buffer_init(options->pool);
    char internal_query[1024] = { '\0' };
    snprintf(internal_query, 1024, "partNumber=%d&uploadId=%s",
             part_number, upload_id);


    make_header_buf(options, PUT_METHOD, bucket, object_key, NULL,
                    -1, internal_query, headers, resp, &err);

    if(NULL != resp->body && resp->status_code < 300){
        oos_get_bucket_etag_parse_from_body(options->pool, etag,  resp->body);
    }

    //oos_str_set(etag,resp->etag);
    parse_status_from_response(resp, &err, status);
    *resp_header = oos_table_clone(options->pool, resp->headers);
    buffer_free(resp);
    return status;
}

oos_status_t* oos_complete_multipart_upload(const request_options_t* options,
                               const char* bucket, const char* object_key,
                               const char* upload_id, part_etags list[], int count,
                               oos_table_t** resp_header) {
    oos_status_t *status = oos_status_create(options->pool);
    if (!bucket || strlen(bucket)<= 0) {
        config_status(STATUS_INVALID_PARAMETER,
            "bucket need not null\n",status);
        return status;
    }

    if (!object_key || strlen(object_key)<= 0) {
        config_status(STATUS_INVALID_PARAMETER,
            "object_key need not null\n",status);
        return status;
    }
    buffer_t *resp = buffer_init(options->pool);

    int err = 0;
    if (upload_id == NULL || strlen(upload_id) <= 0) {
        config_status(STATUS_INVALID_PARAMETER, "upload id is null \n",status);
        return status;
    }
    long buf_size = 5 * 1024 * 1024;
    char* buf = oos_pcalloc(options->pool, buf_size);
    int dat_len = 0;
    int dat_size = buf_size;
    int print_len = 0;
    char *dat = buf;
    
    print_len = snprintf(dat + dat_len, dat_size - dat_len, "<CompleteMultipartUpload>");
    dat_len += print_len;
    int i=0;
    for (i = 1; i <= count; ++i) {
        print_len = snprintf(dat + dat_len, dat_size - dat_len, "<Part><PartNumber>%d</PartNumber><ETag>\"%.*s\"</ETag></Part>",
                             list[i - 1].partnumber, list[i - 1].etag.len, list[i - 1].etag.data);
        dat_len += print_len;
    }
    print_len = snprintf(dat + dat_len, dat_size - dat_len, "</CompleteMultipartUpload>");
    dat_len += print_len;
    printf("CompleteMultipartUpload DATA:\n%s\n", dat);
    
    char internal_query[64] = { '\0' };
    snprintf(internal_query, 64, "uploadId=%s", upload_id);
    
    make_multiparts(options, POST_METHOD, bucket, object_key,
                    dat, dat_len, internal_query, NULL,
                    MULTI_COMPLETE_OP, resp, &err);
    parse_status_from_response(resp, &err, status);
    *resp_header = oos_table_clone(options->pool, resp->headers);
    buffer_free(resp);
    return status;
}

oos_status_t* oos_abort_multipart_upload(const request_options_t* options,
                            const char* bucket, const char* object_key,
                            const char* upload_id,oos_table_t** resp_header) {
    oos_status_t *status = oos_status_create(options->pool);
    buffer_t *resp = buffer_init(options->pool);

    int err = 0;
    if (upload_id == NULL || strlen(upload_id) <= 0) {
        config_status(STATUS_INVALID_PARAMETER, "upload id is null \n",status);
        return status;
    }
    if (object_key == NULL || strlen(object_key) <= 0) {
        config_status(STATUS_INVALID_PARAMETER, "object_key id is null \n",status);
        return status;
    }
    if (!bucket || strlen(bucket)<= 0) {
        config_status(STATUS_INVALID_PARAMETER,
            "bucket need not null\n",status);
        return status;
    }

    char internal_query[64] = { '\0' };
    snprintf(internal_query, 64, "uploadId=%s", upload_id);
    
    make_header(options, DELETE_METHOD, bucket, object_key, NULL, internal_query,
                NULL, resp, &err);
    parse_status_from_response(resp, &err, status);
    *resp_header = oos_table_clone(options->pool, resp->headers);
    buffer_free(resp);
    return status;
}

oos_status_t* oos_list_multipart_uploads(const request_options_t* options, const char* bucket, const char *prefix,
	                                const char *delimiter, const char *max_uploads, const char *key_marker, const char *upload_id_marker,
                                    oos_list_multipart_uploads_t** result,oos_table_t** resp_header) {
   oos_status_t *status = oos_status_create(options->pool);
    if (!bucket || strlen(bucket)<= 0) {
        config_status(STATUS_INVALID_PARAMETER,
            "bucket need not null\n",status);
        return status;
    }

   buffer_t *resp = buffer_init(options->pool);
   int err = 0;
   char query_buf[1024];
   snprintf(query_buf, 1024, "uploads");

   char query[1024] = { '\0' };
   if (prefix && strlen(prefix) > 0) {
	   snprintf(query, 1024, "&prefix=%s", prefix);
	   strcat(query_buf, query);
   }
   if (delimiter && strlen(delimiter) > 0) {
	   snprintf(query, 1024, "&delimiter=%s", delimiter);
	   strcat(query_buf, query);
   }
   if (max_uploads && strlen(max_uploads) > 0) {
	   snprintf(query, 1024, "&max-uploads=%s", max_uploads);
	   strcat(query_buf, query);
   }
   if (key_marker && strlen(key_marker) > 0) {
	   snprintf(query, 1024, "&key-marker=%s", key_marker);
	   strcat(query_buf, query);
   }
   if (upload_id_marker && strlen(upload_id_marker) > 0) {
	   snprintf(query, 1024, "&upload-id-marker=%s", upload_id_marker);
	   strcat(query_buf, query);
   }
   

   make_header(options, GET_METHOD, bucket, NULL, NULL, query_buf, NULL, resp, &err);
   parse_status_from_response(resp, &err, status);
    *resp_header = oos_table_clone(options->pool, resp->headers);
   if(NULL != resp->body && resp->status_code < 300){
        *result = oos_list_multipart_upload_parse_from_body(options->pool, resp->body);     
   }

   buffer_free(resp);
   return status;
}

oos_status_t* oos_list_parts(const request_options_t* options,
                const char* bucket, const char* object_key,
                const char* upload_id, const char* maxparts,
                const char* partnumbermarker, part_listing_t** listing
                ,oos_table_t** resp_header) {
    oos_status_t *status = oos_status_create(options->pool);
    buffer_t *resp = buffer_init(options->pool);

    int err = 0;
    if (upload_id == NULL || strlen(upload_id) <= 0) {
        config_status(STATUS_INVALID_PARAMETER,
            "upload id is null \n",status);
        return status;
    }
    if (object_key == NULL || strlen(object_key) <= 0) {
        config_status(STATUS_INVALID_PARAMETER,
            "object_key id is null \n",status);
        return status;
    }
    char query[1024] = { '\0' };
    char internal_query[1024] = { '\0' };
    snprintf(internal_query, 1024, "uploadId=%s", upload_id);
    if (maxparts && strlen(maxparts) > 0) {
        int imaxparts = atoi(maxparts);
        if (imaxparts > 0 && imaxparts <= 1000) {
            sprintf(query, "&max-parts=%d", imaxparts);
            strcat(internal_query, query);
        }
    }
    if (partnumbermarker && strlen(partnumbermarker) > 0) {
        sprintf(query, "&part-number-marker=%s", partnumbermarker);
        strcat(internal_query, query);
    }
    make_header(options, GET_METHOD, bucket, object_key, NULL, internal_query, NULL,
                resp, &err);
    
    if(NULL != resp->body && resp->status_code < 300){
        *listing = oos_list_parts_parse_from_body(options->pool, resp->body); 
    }

    parse_status_from_response(resp, &err, status);
    *resp_header = oos_table_clone(options->pool, resp->headers);
    buffer_free(resp);
    return status;
}

oos_status_t* oos_get_bucket_location(const request_options_t* options,
                          const char* bucket, location_configuration_t** config
                          ,oos_table_t** resp_header) {

    oos_status_t *status = oos_status_create(options->pool);
    if (!bucket || strlen(bucket)<= 0) {
        config_status(STATUS_INVALID_PARAMETER,
            "bucket need not null\n",status);
        return status;
    }

    buffer_t *resp = buffer_init(options->pool);
    int err = 0;
    char internal_query[16] = { '\0' };
    snprintf(internal_query, 16, "location");

    make_header(options, GET_METHOD, bucket, NULL, NULL, internal_query, NULL,
                resp, &err);
    if(NULL != resp->body && resp->status_code < 300){
        *config =  oos_get_bucket_location_parse_from_body(options->pool, resp->body);
    }
    parse_status_from_response(resp, &err, status);
    *resp_header = oos_table_clone(options->pool, resp->headers);
    buffer_free(resp);
    return status;
}

oos_status_t* oos_get_bucket_acl(const request_options_t* options,
                          const char* bucket, acl_configuration_t** config
                          ,oos_table_t** resp_header) {

    oos_status_t *status = oos_status_create(options->pool);
    if (!bucket || strlen(bucket)<= 0) {
        config_status(STATUS_INVALID_PARAMETER,
            "bucket need not null\n",status);
        return status;
    }

    buffer_t *resp = buffer_init(options->pool);
    int err = 0;
    char internal_query[16] = { '\0' };
    snprintf(internal_query, 16, "acl");

    make_header(options, GET_METHOD, bucket, NULL, NULL, internal_query, NULL,
                resp, &err);
    if(NULL != resp->body && resp->status_code < 300){
        *config =  oos_get_bucket_acl_parse_from_body(options->pool, resp->body);
    }
    parse_status_from_response(resp, &err, status);
    *resp_header = oos_table_clone(options->pool, resp->headers);
    buffer_free(resp);
    return status;
}

oos_status_t* oos_get_regions(const request_options_t* options,
                          region_configuration_t** config
                          ,oos_table_t** resp_header) {

    oos_status_t *status = oos_status_create(options->pool);

    buffer_t *resp = buffer_init(options->pool);
    int err = 0;
    char internal_query[16] = { '\0' };
    snprintf(internal_query, 16, "regions");

    make_header(options, GET_METHOD, NULL, NULL, NULL, internal_query, NULL,
                resp, &err);
    if(NULL != resp->body && resp->status_code < 300){
        *config =  oos_get_region_parse_from_body(options->pool, resp->body);
    }
    parse_status_from_response(resp, &err, status);
    *resp_header = oos_table_clone(options->pool, resp->headers);
    buffer_free(resp);
    return status;
}
oos_status_t* oos_put_bucket_accelerate(const request_options_t* options,
                          const char* bucket, accelerate_configuration_t* config
                          ,oos_table_t** resp_header) {
    oos_status_t *status = oos_status_create(options->pool);
    if (!bucket || strlen(bucket)<= 0) {
        config_status(STATUS_INVALID_PARAMETER,
            "bucket need not null\n",status);
        return status;
    }

    if (!config ) {
        config_status(STATUS_INVALID_PARAMETER,
            "config need not null\n",status);
        return status;
    }

    buffer_t *resp = buffer_init(options->pool);
    int err = 0;

    long buf_size = 5 * 1024 * 1024;
    char* buf = (char *)oos_pcalloc(options->pool,buf_size);
    int dat_len = 0;
    int dat_size = buf_size;
    int print_len = 0;
    char *dat = buf;

    // status
    char accelerate_status[64+1] = {"\0"};
    if(ENABLED == config->status){
        strcpy(accelerate_status, "Enabled");
    }else{
        strcpy(accelerate_status, "Suspended");
    }

    print_len = snprintf(dat + dat_len, dat_size - dat_len, "<AccelerateConfiguration><Status>%s</Status><IPWhiteLists>",
                         accelerate_status);
    dat_len += print_len;

    // ip white list
    list_string_node_t* tmp;
    list_for_each_entry(tmp, &config->ip_white_list.node, node)
    {
        print_len = snprintf(dat + dat_len, dat_size - dat_len, "<IP>%s</IP>",
                             tmp->text.data);
        dat_len += print_len;
    }

    print_len = snprintf(dat + dat_len, dat_size - dat_len, "</IPWhiteLists></AccelerateConfiguration>");
    dat_len += print_len;

    printf("accelerater is:%s\n",dat);

    char header_str[1024];
    char  base64_buf[32];
    compute_buf_md5b64(dat, dat_len, base64_buf);
    snprintf(header_str, 128, "Content-Md5: %s", base64_buf);

    char internal_query[16] = { '\0' };
    snprintf(internal_query, 16, "accelerate");

    make_lifecycle(options, PUT_METHOD, bucket, NULL,
                   dat, dat_len, internal_query, header_str,
                   resp, &err);
    parse_status_from_response(resp, &err, status);
    *resp_header = oos_table_clone(options->pool, resp->headers);
    buffer_free(resp);
    return status;
}

oos_status_t* oos_put_bucket_cors(const request_options_t* options,
                          const char* bucket, cors_configuration_t* config
                          ,oos_table_t** resp_header) {
    oos_status_t *status = oos_status_create(options->pool);
    if (!bucket || strlen(bucket)<= 0) {
        config_status(STATUS_INVALID_PARAMETER,
            "bucket need not null\n",status);
        return status;
    }

    if (!config ) {
        config_status(STATUS_INVALID_PARAMETER,
            "config need not null\n",status);
        return status;
    }

    buffer_t *resp = buffer_init(options->pool);
    int err = 0;

    long buf_size = 5 * 1024 * 1024;
    char* buf = (char *)oos_pcalloc(options->pool,buf_size);
    int dat_len = 0;
    int dat_size = buf_size;
    int print_len = 0;
    char *dat = buf;

    // start
    print_len = snprintf(dat + dat_len, dat_size - dat_len, "<CORSConfiguration>");
    dat_len += print_len;

    int count_cors = 0, count_orign = 0, count_method = 0;
    // core_rule
    list_cors_rule_node_t* cors_rule_node;
    list_for_each_entry(cors_rule_node, &config->cors_rule_list.node, node)
    {
        count_cors ++;
        print_len = snprintf(dat + dat_len, dat_size - dat_len, "<CORSRule>");
           dat_len += print_len;

        // id
        if(cors_rule_node->id.len > 0){
            print_len = snprintf(dat + dat_len, dat_size - dat_len, "<ID>%s</ID>",
                                 cors_rule_node->id.data);
            dat_len += print_len;
        }

        // allowed orign list
        list_string_node_t* orign_node;
        list_for_each_entry(orign_node, &cors_rule_node->allowed_origin_list.node, node)
        {
            print_len = snprintf(dat + dat_len, dat_size - dat_len, "<AllowedOrigin>%s</AllowedOrigin>",
                                 orign_node->text.data);
            dat_len += print_len;
            count_orign ++;
        }
        // allowed method list
        list_string_node_t* method_node;
        list_for_each_entry(method_node, &cors_rule_node->allowed_method_list.node, node)
        {
            print_len = snprintf(dat + dat_len, dat_size - dat_len, "<AllowedMethod>%s</AllowedMethod>",
                                 method_node->text.data);
            dat_len += print_len;
            count_method ++;
        }
        // allowed header list
        list_string_node_t* header_node;
        list_for_each_entry(header_node, &cors_rule_node->allowed_header_list.node, node)
        {
            print_len = snprintf(dat + dat_len, dat_size - dat_len, "<AllowedHeader>%s</AllowedHeader>",
                                 header_node->text.data);
            dat_len += print_len;
        }
        // expose header list
        list_string_node_t* expose_header_node;
        list_for_each_entry(expose_header_node, &cors_rule_node->expose_header_list.node, node)
        {
            print_len = snprintf(dat + dat_len, dat_size - dat_len, "<ExposeHeader>%s</ExposeHeader>",
                                 expose_header_node->text.data);
            dat_len += print_len;
        }
        // MaxAgeSeconds
        if(cors_rule_node->maxage_seconds > 0){
            print_len = snprintf(dat + dat_len, dat_size - dat_len, "<MaxAgeSeconds>%d</MaxAgeSeconds>",
                                 cors_rule_node->maxage_seconds);
            dat_len += print_len;
        }

        print_len = snprintf(dat + dat_len, dat_size - dat_len, "</CORSRule>");
        dat_len += print_len;
    }

    if(count_cors < 1){
        config_status(STATUS_INVALID_PARAMETER,
            "cors rule  need not null\n",status);
        return status;
    }

    if(count_orign < 1){
        config_status(STATUS_INVALID_PARAMETER,
            "AllowedOrign  need not null\n",status);
        return status;
    }

    if(count_method < 1){
        config_status(STATUS_INVALID_PARAMETER,
            "AllowedMetho  need not null\n",status);
        return status;
    }

    // end
    print_len = snprintf(dat + dat_len, dat_size - dat_len, "</CORSConfiguration>");
    dat_len += print_len;

    printf("accelerater is:%s\n",dat);

    char header_str[1024];
    char  base64_buf[32];
    compute_buf_md5b64(dat, dat_len, base64_buf);
    snprintf(header_str, 128, "Content-Md5: %s", base64_buf);

    char internal_query[16] = { '\0' };
    snprintf(internal_query, 16, "cors");

    make_lifecycle(options, PUT_METHOD, bucket, NULL,
                   dat, dat_len, internal_query, header_str,
                   resp, &err);
    parse_status_from_response(resp, &err, status);
    *resp_header = oos_table_clone(options->pool, resp->headers);
    buffer_free(resp);
    return status;
}
oos_status_t* oos_get_bucket_cors(const request_options_t* options,
                          const char* bucket,
                          cors_configuration_t** config
                          ,oos_table_t** resp_header) {

    oos_status_t *status = oos_status_create(options->pool);

    if (!bucket || strlen(bucket)<= 0) {
        config_status(STATUS_INVALID_PARAMETER,
            "bucket need not null\n",status);
        return status;
    }

    buffer_t *resp = buffer_init(options->pool);
    int err = 0;
    char internal_query[16] = { '\0' };
    snprintf(internal_query, 16, "cors");

    make_header(options, GET_METHOD, bucket, NULL, NULL, internal_query, NULL,
                resp, &err);
    if(NULL != resp->body && resp->status_code < 300){
        *config =  oos_get_cors_parse_from_body(options->pool, resp->body);
    }
    parse_status_from_response(resp, &err, status);
    *resp_header = oos_table_clone(options->pool, resp->headers);
    buffer_free(resp);
    return status;
}

oos_status_t* oos_delete_bucket_cors(const request_options_t* options,
                             const char* bucket,oos_table_t** resp_header) {
    oos_status_t *status = oos_status_create(options->pool);
    if (!bucket || strlen(bucket)<= 0) {
        config_status(STATUS_INVALID_PARAMETER,
            "bucket need not null\n",status);
        return status;
    }
    buffer_t *resp = buffer_init(options->pool);

    int err = 0;
    char internal_query[16] = { '\0' };
    snprintf(internal_query, 16, "cors");

    make_header(options, DELETE_METHOD, bucket, NULL, NULL, internal_query, NULL,
                resp, &err);
    parse_status_from_response(resp, &err, status);
    *resp_header = oos_table_clone(options->pool, resp->headers);
    buffer_free(resp);
    return status;
}

oos_status_t* oos_get_bucket_accelerate(const request_options_t* options,
                          const char* bucket, accelerate_configuration_t** config
                          ,oos_table_t** resp_header) {
    oos_status_t *status = oos_status_create(options->pool);
    if (!bucket || strlen(bucket)<= 0) {
        config_status(STATUS_INVALID_PARAMETER,
            "bucket need not null\n",status);
        return status;
    }

    buffer_t *resp = buffer_init(options->pool);
    int err = 0;
    char internal_query[16] = { '\0' };
    snprintf(internal_query, 16, "accelerate");

    make_header(options, GET_METHOD, bucket, NULL, NULL, internal_query, NULL,
                resp, &err);
    if(NULL != resp->body && resp->status_code < 300){
        *config =  oos_get_bucket_accelerate_parse_from_body(options->pool, resp->body);
    }
    parse_status_from_response(resp, &err, status);
    *resp_header = oos_table_clone(options->pool, resp->headers);
    buffer_free(resp);
    return status;
}

oos_status_t* oos_put_bucket_lifecycle(const request_options_t* options, 
                          const char* bucket, lifecycle_configuration_t config[], int count
                          ,oos_table_t** resp_header) {
    oos_status_t *status = oos_status_create(options->pool);
    if (!bucket || strlen(bucket)<= 0) {
        config_status(STATUS_INVALID_PARAMETER,
            "bucket need not null\n",status);
        return status;
    }
    buffer_t *resp = buffer_init(options->pool);
    int err = 0;

    long buf_size = 5 * 1024 * 1024;
    char* buf = (char *)oos_pcalloc(options->pool,buf_size);
    int dat_len = 0;
    int dat_size = buf_size;
    int print_len = 0;
    char *dat = buf;
    
    print_len = snprintf(dat + dat_len, dat_size - dat_len, "<LifecycleConfiguration>");
    dat_len += print_len;
    int i; 
    for (i = 0; i < count; i++) {
        char s_prefix[512] = {"\0"};
        char s_ruleid[512] = {"\0"};
        char s_status[512] = {"\0"};
        if(!oos_string_is_empty(&config[i].prefix)){
           strncpy(s_prefix, config[i].prefix.data, config[i].prefix.len > sizeof(s_prefix)?sizeof(s_prefix):config[i].prefix.len); 
        }
        if(!oos_string_is_empty(&config[i].ruleid)){
           strncpy(s_ruleid, config[i].ruleid.data, config[i].ruleid.len > sizeof(s_ruleid)?sizeof(s_ruleid):config[i].ruleid.len);
        }
        if(!oos_string_is_empty(&config[i].status)){
           strncpy(s_status, config[i].status.data, config[i].status.len > sizeof(s_status)?sizeof(s_status):config[i].status.len);
        }

        if(config[i].days > 0)
        {
            print_len = snprintf(dat + dat_len, dat_size - dat_len, "<Rule><ID>%s</ID><Prefix>%s</Prefix><Status>%s</Status><Expiration><Days>%d</Days></Expiration></Rule>",
                             s_ruleid, s_prefix, s_status,config[i].days);
            dat_len += print_len;
        }
        else if(!oos_string_is_empty(&config[i].date))
        {
            print_len = snprintf(dat + dat_len, dat_size - dat_len, "<Rule><ID>%s</ID><Prefix>%s</Prefix><Status>%s</Status><Expiration><Date>%s</Date></Expiration></Rule>",
                              s_ruleid, s_prefix , s_status , config[i].date.data);
            dat_len += print_len;
        }else{
            config_status(STATUS_INVALID_PARAMETER,
            "day or date must choose one \n",status);
            return status;
        }
    }
    print_len = snprintf(dat + dat_len, dat_size - dat_len, "</LifecycleConfiguration>");
    dat_len += print_len;
    printf("lifecycle is:%s\n",dat);
    
    char header_str[1024];
    char  base64_buf[32];
    compute_buf_md5b64(dat, dat_len, base64_buf);
    snprintf(header_str, 128, "Content-Md5: %s", base64_buf);
    
    char internal_query[16] = { '\0' };
    snprintf(internal_query, 16, "lifecycle");
    
    make_lifecycle(options, PUT_METHOD, bucket, NULL,
                   dat, dat_len, internal_query, header_str,
                   resp, &err);
    parse_status_from_response(resp, &err, status);
    *resp_header = oos_table_clone(options->pool, resp->headers);
    buffer_free(resp);
    return status;
}

oos_status_t* oos_get_bucket_lifecycle(const request_options_t* options,
                          const char* bucket, lifecycle_configuration_t** config
                          ,oos_table_t** resp_header) {
    oos_status_t *status = oos_status_create(options->pool);
    if (!bucket || strlen(bucket)<= 0) {
        config_status(STATUS_INVALID_PARAMETER,
            "bucket need not null\n",status);
        return status;
    }

    buffer_t *resp = buffer_init(options->pool);
    int err = 0;
    char internal_query[16] = { '\0' };
    snprintf(internal_query, 16, "lifecycle");
    
    make_header(options, GET_METHOD, bucket, NULL, NULL, internal_query, NULL,
                resp, &err);
    if(NULL != resp->body && resp->status_code < 300){
        *config =  oos_get_bucket_lifecycle_parse_from_body(options->pool, resp->body);
    }
    parse_status_from_response(resp, &err, status);
    *resp_header = oos_table_clone(options->pool, resp->headers);
    buffer_free(resp);
    return status;
}

oos_status_t* oos_delete_bucket_lifecycle(const request_options_t* options, 
                             const char* bucket,oos_table_t** resp_header) {
    oos_status_t *status = oos_status_create(options->pool);   
    if (!bucket || strlen(bucket)<= 0) {
        config_status(STATUS_INVALID_PARAMETER,
            "bucket need not null\n",status);
        return status;
    }
    buffer_t *resp = buffer_init(options->pool);

    int err = 0;
    char internal_query[16] = { '\0' };
    snprintf(internal_query, 16, "lifecycle");
    
    make_header(options, DELETE_METHOD, bucket, NULL, NULL, internal_query, NULL,
                resp, &err);
    parse_status_from_response(resp, &err, status);
    *resp_header = oos_table_clone(options->pool, resp->headers);
    buffer_free(resp);
    return status;
}

oos_status_t* oos_put_bucket_website(const request_options_t* options,
                        const char* bucket, website_configuration_t* config
                        ,oos_table_t** resp_header){
    oos_status_t *status = oos_status_create(options->pool);
    buffer_t *resp = buffer_init(options->pool);

    if (!bucket || strlen(bucket)<= 0) {
        config_status(STATUS_INVALID_PARAMETER,
            "bucket need not null\n",status);
        return status;
    }

    int err = 0;
    long buf_size = 5 * 1024 * 1024;
    char* buf = (char *)oos_pcalloc(options->pool,buf_size);
    
    int dat_len = 0;
    int dat_size = buf_size;
    int print_len = 0;
    char *dat = buf;
    
    print_len = snprintf(dat + dat_len, dat_size - dat_len, "<WebsiteConfiguration>");
    dat_len += print_len;
    if(!oos_string_is_empty(&config->suffix)){
        print_len = snprintf(dat + dat_len, dat_size - dat_len, "<IndexDocument><Suffix>%s</Suffix></IndexDocument>",
                             config->suffix.data);
        dat_len += print_len;
    }
    if(!oos_string_is_empty(&config->key)){
        print_len = snprintf(dat + dat_len, dat_size - dat_len, "<ErrorDocument><Key>%s</Key></ErrorDocument>",
                             config->key.data);
        dat_len += print_len;
    }
    print_len = snprintf(dat + dat_len, dat_size - dat_len, "</WebsiteConfiguration>");
    dat_len += print_len;
    printf("website is:%s\n",dat);
    
    char header_str[1024];
    char  base64_buf[32];
    compute_buf_md5b64(dat, dat_len, base64_buf);
    snprintf(header_str, 1024, "Content-Md5: %s", base64_buf);
    
    char internal_query[16] = { '\0' };
    snprintf(internal_query, 16, "website");

    make_lifecycle(options, PUT_METHOD, bucket, NULL,
                   dat, dat_len, internal_query, header_str,
                   resp, &err);

    parse_status_from_response(resp, &err, status);
    *resp_header = oos_table_clone(options->pool, resp->headers);
    buffer_free(resp);
    return status;
}


oos_status_t* oos_get_bucket_website(const request_options_t* options,
                        const char* bucket, website_configuration_t** config
                        ,oos_table_t** resp_header){
    oos_status_t *status = oos_status_create(options->pool);
    if (!bucket || strlen(bucket)<= 0) {
        config_status(STATUS_INVALID_PARAMETER,
            "bucket need not null\n",status);
        return status;
    }
    buffer_t *resp = buffer_init(options->pool);
    int err = 0;

    char internal_query[16] = { '\0' };
    snprintf(internal_query, 16, "website");
    make_header(options, GET_METHOD, bucket, NULL, NULL, internal_query, NULL,
                resp, &err);
    if(NULL != resp->body && resp->status_code < 300){
        *config = oos_get_bucket_website_parse_from_body(options->pool, resp->body);
    }
    parse_status_from_response(resp, &err, status);
    *resp_header = oos_table_clone(options->pool, resp->headers);
    buffer_free(resp);
    return status;
}

oos_status_t* oos_delete_bucket_website(const request_options_t* options,
                           const char* bucket,oos_table_t** resp_header){
    oos_status_t *status = oos_status_create(options->pool);
    if (!bucket || strlen(bucket)<= 0) {
        config_status(STATUS_INVALID_PARAMETER,
            "bucket need not null\n",status);
        return status;
    }
    buffer_t *resp = buffer_init(options->pool);
    int err = 0;

    char internal_query[16] = { '\0' };
    snprintf(internal_query, 16, "website");
    
    make_header(options, DELETE_METHOD, bucket, NULL, NULL, internal_query, NULL,
               resp, &err);
    parse_status_from_response(resp, &err, status);
    *resp_header = oos_table_clone(options->pool, resp->headers);
    buffer_free(resp);
    return status;
}

oos_status_t* oos_put_bucket_logging(const request_options_t* options,
                       const char* bucket, logging_configuration_t* config
                       ,oos_table_t** resp_header){
    oos_status_t *status = oos_status_create(options->pool);
    if (!bucket || strlen(bucket)<= 0) {
        config_status(STATUS_INVALID_PARAMETER,
            "bucket need not null\n",status);
        return status;
    }
    buffer_t *resp = buffer_init(options->pool);

    int err = 0;
    long buf_size = 5 * 1024 * 1024;
    char* buf = (char *)oos_pcalloc(options->pool,buf_size);
    int dat_len = 0;
    int dat_size = buf_size;
    int print_len = 0;
    char *dat = buf;
    
    print_len = snprintf(dat + dat_len, dat_size - dat_len, "<BucketLoggingStatus><LoggingEnabled>");
    dat_len += print_len;
    if(!oos_string_is_empty(&config->targetBucket)){
        print_len = snprintf(dat + dat_len, dat_size - dat_len, "<TargetBucket>%s</TargetBucket>",
                             config->targetBucket.data);
        dat_len += print_len;
    }
    if(!oos_string_is_empty(&config->targetPrefix)){
        print_len = snprintf(dat + dat_len, dat_size - dat_len, "<TargetPrefix>%s</TargetPrefix>",
                             config->targetPrefix.data);
        dat_len += print_len;
    }
    if(!oos_string_is_empty(&config->triggerTargetBucket)){
        print_len = snprintf(dat + dat_len, dat_size - dat_len, "<TriggerTargetBucket>%s</TriggerTargetBucket>",
                             config->triggerTargetBucket.data);
        dat_len += print_len;
    }
    if(!oos_string_is_empty(&config->triggerTargetPrefix)){
        print_len = snprintf(dat + dat_len, dat_size - dat_len, "<TriggerTargetPrefix>%s</TriggerTargetPrefix>",
                             config->triggerTargetPrefix.data);
        dat_len += print_len;
    }
    if(!oos_string_is_empty(&config->triggerSourceBucket)){
        print_len = snprintf(dat + dat_len, dat_size - dat_len, "<TriggerSourceBucket>%s</TriggerSourceBucket>",
                             config->triggerSourceBucket.data);
        dat_len += print_len;
    }
    if(!oos_string_is_empty(&config->triggerSourcePrefix)){
        print_len = snprintf(dat + dat_len, dat_size - dat_len, "<TriggerSourcePrefix>%s</TriggerSourcePrefix>",
                             config->triggerSourcePrefix.data);
        dat_len += print_len;
    }
    print_len = snprintf(dat + dat_len, dat_size - dat_len, "</LoggingEnabled></BucketLoggingStatus>");
    dat_len += print_len;
    printf("logging is:%s\n",dat);
    
    char header_str[1024];
    char  base64_buf[32];
    compute_buf_md5b64(dat, dat_len, base64_buf);
    snprintf(header_str, 1024, "Content-Md5: %s", base64_buf);
    
    char internal_query[16] = { '\0' };
    snprintf(internal_query, 16, "logging");

    make_lifecycle(options, PUT_METHOD, bucket, NULL,
                   dat, dat_len, internal_query, header_str,
                   resp, &err);
    parse_status_from_response(resp, &err, status);
    *resp_header = oos_table_clone(options->pool, resp->headers);
    buffer_free(resp);
    return status;
}

oos_status_t* oos_get_bucket_logging(const request_options_t* options,
                        const char* bucket, logging_configuration_t** config
                        ,oos_table_t** resp_header){
    oos_status_t *status = oos_status_create(options->pool);
    if (!bucket || strlen(bucket)<= 0) {
        config_status(STATUS_INVALID_PARAMETER,
            "bucket need not null\n",status);
        return status;
    }
    buffer_t *resp = buffer_init(options->pool);

    int err = 0;
    char internal_query[16] = { '\0' };
    snprintf(internal_query, 16, "logging");
    make_header(options, GET_METHOD, bucket, NULL, NULL, internal_query, NULL,
               resp, &err);
    if(NULL != resp->body && resp->status_code < 300){
        *config = oos_get_bucket_logging_parse_from_body(options->pool, resp->body);
    }
    parse_status_from_response(resp, &err, status);
    *resp_header = oos_table_clone(options->pool, resp->headers);
    buffer_free(resp);
    return status;
}

oos_status_t* oos_put_bucket_policy(const request_options_t* options,
                       const char* bucket, char* policy_text,oos_table_t** resp_header){
    oos_status_t *status = oos_status_create(options->pool);
    if (!bucket || strlen(bucket)<= 0) {
        config_status(STATUS_INVALID_PARAMETER,
            "bucket need not null\n",status);
        return status;
    }
    buffer_t *resp = buffer_init(options->pool);

    int err = 0;
    int dat_len = strlen(policy_text);
    char *dat = policy_text;
    
    printf("policy text is:%s\n",dat);
    
    char header_str[1024];
    char base64_buf[32];
    compute_buf_md5b64(policy_text, dat_len, base64_buf);
    snprintf(header_str, 1024, "Content-Md5: %s", base64_buf);
    
    char internal_query[16] = { '\0' };
    snprintf(internal_query, 16, "policy");

    make_lifecycle(options, PUT_METHOD, bucket, NULL,
                   dat, dat_len, internal_query, header_str,
                   resp, &err);
    parse_status_from_response(resp, &err, status);
    *resp_header = oos_table_clone(options->pool, resp->headers);
    buffer_free(resp);
    return status;  
}

oos_status_t* oos_get_bucket_policy(const request_options_t* options,
                       const char* bucket, char* policy_text,oos_table_t** resp_header){
    oos_status_t *status = oos_status_create(options->pool);
    if (!bucket || strlen(bucket)<= 0) {
        config_status(STATUS_INVALID_PARAMETER,
            "bucket need not null\n",status);
        return status;
    }
    buffer_t *resp = buffer_init(options->pool);
    int err = 0;
    char internal_query[16] = { '\0' };
    snprintf(internal_query, 16, "policy");
    make_header(options, GET_METHOD, bucket, NULL, NULL, internal_query, NULL,
                resp, &err);
    if(resp->status_code < 300 && resp && resp->body)
    {
        snprintf(policy_text, strlen(resp->body),resp->body);
    }
    parse_status_from_response(resp, &err, status);
    *resp_header = oos_table_clone(options->pool, resp->headers);
    buffer_free(resp);
    return status;   
}

oos_status_t* oos_delete_bucket_policy(const request_options_t* options,
                          const char* bucket, oos_table_t** resp_header){
    oos_status_t *status = oos_status_create(options->pool);
    if (!bucket || strlen(bucket)<= 0) {
        config_status(STATUS_INVALID_PARAMETER,
            "bucket need not null\n",status);
        return status;
    }
    buffer_t *resp = buffer_init(options->pool);
    int err = 0;
    char internal_query[16] = { '\0' };
    snprintf(internal_query, 16, "policy");
    
    make_header(options, DELETE_METHOD, bucket, NULL, NULL, internal_query, NULL,
                resp, &err);
    parse_status_from_response(resp, &err, status);
    *resp_header = oos_table_clone(options->pool, resp->headers);
    buffer_free(resp);
    return status;
}

oos_status_t* oos_create_ctyun_access_key(const request_options_t* options,
                        oos_list_access_key_member_t** access_member,oos_table_t** resp_header){
    oos_status_t *status = oos_status_create(options->pool);
    buffer_t *resp = buffer_init(options->pool);
    int err = 0;
    char body[128] = { '\0' };
    snprintf(body, 128, "Action=CreateAccessKey");
    make_header_post(options, POST_METHOD, NULL, NULL, body, NULL, NULL,
                resp, &err);
    if(NULL != resp->body && resp->status_code < 300){
       *access_member = oos_create_access_key_parse_from_body(options->pool, resp->body);
    }
    parse_status_from_response(resp, &err, status);
    *resp_header = oos_table_clone(options->pool, resp->headers);
    buffer_free(resp);
    printf("status %p\n", status);
    return status;
}

oos_status_t* oos_ctyun_delete_access_key(const request_options_t* options,
                             const char* deleted_accessKey, oos_table_t** resp_header){
    oos_status_t *status = oos_status_create(options->pool);
    buffer_t *resp = buffer_init(options->pool);

    int err = 0;
    char body[128] = { '\0' };
    snprintf(body, 128, "Action=DeleteAccessKey&AccessKeyId=%s",deleted_accessKey);
    make_header_post(options, POST_METHOD, NULL, NULL, body, NULL, NULL,
                resp, &err);
    parse_status_from_response(resp, &err, status);
    *resp_header = oos_table_clone(options->pool, resp->headers);
    buffer_free(resp);
    return status;
}

oos_status_t* oos_ctyun_update_access_key(const request_options_t* options,
                             const char* updated_accessKey,const char* status,
                             const char* is_primary,oos_table_t** resp_header){
    oos_status_t *oos_status = oos_status_create(options->pool);
    buffer_t *resp = buffer_init(options->pool);
    int err = 0;
    char body[128] = { '\0' };
    snprintf(body, 128, "Action=UpdateAccessKey&AccessKeyId=%s&Status=%s&IsPrimary=%s",
        updated_accessKey,status,is_primary);
    make_header_post(options, POST_METHOD, NULL, NULL, body, NULL, NULL,
                resp, &err);
    parse_status_from_response(resp, &err, oos_status);
    buffer_free(resp);
    return oos_status;
}

oos_status_t* oos_ctyun_list_access_key(const request_options_t* options,
                           int maxItems, const char* marker,oos_list_access_key_t** keys
                           ,oos_table_t** resp_header){
    oos_status_t *oos_status = oos_status_create(options->pool);
    buffer_t *resp = buffer_init(options->pool);
    int err = 0;
    if(maxItems <= 0){
        maxItems = 100;  //默认返回 100 个 key
    }
    char body[128] = { '\0' };
    if(!marker)
        snprintf(body, 128, "Action=ListAccessKey&MaxItems=%d", maxItems);
    else
        snprintf(body, 128, "Action=ListAccessKey&MaxItems=%d&Marker=%s", maxItems, marker);

    make_header_post(options, POST_METHOD, NULL, NULL, body, NULL, NULL,
                resp, &err);
    if(NULL != resp->body && resp->status_code < 300){
        *keys = oos_list_access_parse_from_body(options->pool,resp->body);
    }
    parse_status_from_response(resp, &err, oos_status);
    buffer_free(resp);
    return oos_status;
}

oos_status_t* oos_ctyun_create_get_uasge(const request_options_t* options,const char* date,
                              const oos_string_t pools[],const int count, const char* details
                              ,oos_table_t** resp_header){
    oos_status_t *status = oos_status_create(options->pool);
    buffer_t *resp = buffer_init(options->pool);
    int err = 0;
    char body[1024] = { '\0' };

    long buf_size = 16 * 1024 * 1024;
    char* dat = oos_pcalloc(options->pool, buf_size);
    int dat_len = 0;
    int dat_size = buf_size;
    int print_len = 0;
    print_len = snprintf(dat + dat_len, dat_size - dat_len, "Action=GetUsage");
    dat_len += print_len;
    if(strcmp(date,"")!=0){
        print_len = snprintf(dat + dat_len, dat_size - dat_len, "&Date=%s",date);
        dat_len += print_len;
    }

    if(count > 0){//123+abc
        print_len = snprintf(dat + dat_len, dat_size - dat_len, "&Pools=");
        dat_len += print_len;
        int i;
        for(i =0 ;i< count; i++){
            print_len = snprintf(dat + dat_len, dat_size - dat_len, "%s+",
                             pools[i].data);
            dat_len += print_len;
        }
    }
    dat_len = dat_len - 1;
    if(strcmp(details,"")!=0){
        print_len = snprintf(dat + dat_len, dat_size - dat_len, "&Details=%s",details);
    }else{
        print_len = snprintf(dat + dat_len, dat_size - dat_len, "&Details=False");
    }
    dat_len += print_len;
    printf("----- dat %s\n", dat);

    make_header_post(options, POST_METHOD, NULL, NULL, dat, NULL, NULL,
                resp, &err);
    parse_status_from_response(resp, &err, status);
    *resp_header = oos_table_clone(options->pool, resp->headers);
    buffer_free(resp);

    return status;
}

void parse_meta_data(const oos_pool_t *pool, const char *headers, oos_table_t** meta_data){
    *meta_data = oos_table_make(pool, 1);

    char * pLineTmp = NULL, *pKeyTmp, *line = NULL;
    char *key = NULL, * value = NULL;
    char *pbuf = (char*)headers;

    while(NULL != (line = strtok_r(pbuf, "\r\n", &pLineTmp)))
    {
        //printf(" line  :%s\n", line);
        pbuf = NULL;
        key = strtok_r(line, ":", &pKeyTmp);
        if(key){
            value = strtok_r(NULL, ":", &pKeyTmp);
            if(strncmp(key, "x-amz", 5) == 0){
               oos_table_set(*meta_data, key, value+1);
            }
            // printf("key:%s, value:%s\n", key, value);
        }
    }
}

