#include "oos_string.h"

typedef int (*oos_is_char_pt)(char c);

static void oos_strip_str_func(oos_string_t *str, oos_is_char_pt func);

char *oos_pstrdup(oos_pool_t *p, const oos_string_t *s)
{
    return apr_pstrndup(p, s->data, s->len);
}

static void oos_strip_str_func(oos_string_t *str, oos_is_char_pt func)
{
    char *data = str->data;
    int len = str->len;
    int offset = 0;

    if (len == 0) return;
    
    while (len > 0 && func(data[len - 1])) {
        --len;
    }
    
    for (; offset < len && func(data[offset]); ++offset) {
        // empty;
    }

    str->data = data + offset;
    str->len = len - offset;
}

void oos_unquote_str(oos_string_t *str)
{
    oos_strip_str_func(str, oos_is_quote);
}

void oos_strip_space(oos_string_t *str)
{
    oos_strip_str_func(str, oos_is_space);
}

void oos_trip_space_and_cntrl(oos_string_t *str)
{
    oos_strip_str_func(str, oos_is_space_or_cntrl);
}

int oos_ends_with(const oos_string_t *str, const oos_string_t *suffix)
{
    if (!str || !suffix) {
        return 0;
    }

    return (str->len >= suffix->len) && strncmp(str->data + str->len - suffix->len, suffix->data, suffix->len) == 0;
}

