#ifndef _OOS_SDK_C_STRING_H_
#define _OOS_SDK_C_STRING_H_

#include "oos_define.h"

typedef struct {
    int64_t len;
    char *data;
} oos_string_t;

#define oos_string(str)     { sizeof(str) - 1, (char *) str }
#define oos_null_string     { 0, NULL }
//#define oos_str_set(str, text)                                  \
//    if(text){(str)->len = strlen(text); (str)->data = (char *) text;}   \
//    else{(str)->len = 0; (str)->data = NULL;}
#define oos_str_null(str)   (str)->len = 0; (str)->data = NULL

#define oos_tolower(c)      (char) ((c >= 'A' && c <= 'Z') ? (c | 0x20) : c)
#define oos_toupper(c)      (char) ((c >= 'a' && c <= 'z') ? (c & ~0x20) : c)

static APR_INLINE void oos_str_set(oos_pool_t *p, oos_string_t * str, char* text){
    if(!p || !str || !text){
        return ;
    }
    if(strlen(text)>0){
        str->len = strlen(text);
        str->data =  oos_palloc(p, str->len+1);
        strcpy(str->data, text);
    }
}

static APR_INLINE int oos_string_is_empty(oos_string_t *str)
{
    if (NULL == str || str->len == 0  || NULL == str->data || 0 == strcmp(str->data, "")) {
        return OOS_TRUE;
    } else {
        return OOS_FALSE;
    } 
}

static APR_INLINE void oos_string_tolower(oos_string_t *str)
{
    int i = 0;
    while (i < str->len) {
        str->data[i] = oos_tolower(str->data[i]);
        ++i;
    }
}

static APR_INLINE char *oos_strlchr(char *p, char *last, char c)
{
    while (p < last) {
        if (*p == c) {
            return p;
        }
        p++;
    }
    return NULL;
}

static APR_INLINE int oos_is_quote(char c)
{
    return c == '\"';
}

static APR_INLINE int oos_is_space(char c)
{
    return ((c == ' ') || (c == '\t'));
}

static APR_INLINE int oos_is_space_or_cntrl(char c)
{
    return c <= ' ';
}

static APR_INLINE int oos_is_null_string(oos_string_t *str)
{
    if (str == NULL || str->data == NULL || str->len == 0) {
        return OOS_TRUE;
    }
    return OOS_FALSE;
}

void oos_strip_space(oos_string_t *str);
void oos_trip_space_and_cntrl(oos_string_t *str);
void oos_unquote_str(oos_string_t *str);

char *oos_pstrdup(oos_pool_t *p, const oos_string_t *s);

int oos_ends_with(const oos_string_t *str, const oos_string_t *suffix);


#endif

