
#include <stdio.h>
#include <time.h>
#include <malloc.h>
#include <string.h>
#include <ctype.h>
#include "sign.h"
#include "make_header.h"
#include "hmacsha256.h"
#include "util.h"



static const char* SUB_RESOURCES[17]={"acl", "lifecycle", "location", "logging", "notification", "partNumber", \
	"policy", "requestPayment", "torrent", "uploadId", "uploads", "versionId", \
    "versioning", "versions", "website", "cors", "delete"};

static const char* SUB_RESOURCES_V4[28]={"acl", "lifecycle", "location", "logging", "notification", "partNumber", \
    "policy", "requestPayment", "torrent", "uploadId", "uploads", "versionId", \
    "versioning", "versions", "website", "regions", "accelerate", "cors", "delete", "prefix","delimiter", "marker", "max-keys", \
	"max-parts", "part-number-marker", "max-uploads", "key-marker", "upload-id-marker"};

static const char* RESPONSE_OVERIDES[6]={"response-content-type", "response-content-language", \
	"response-expires", "response-cache-control", \
	"response-content-disposition", "response-content-encoding"};

static unsigned int num_of_sub   = sizeof(SUB_RESOURCES)/sizeof(char*);
static unsigned int num_of_sub_v4   = sizeof(SUB_RESOURCES_V4)/sizeof(char*);
static unsigned int num_of_rides = sizeof(RESPONSE_OVERIDES)/sizeof(char*);

static unsigned char hexchars[] = "0123456789ABCDEF";
static const char* specify_header = "x-amz";
//static const char* specify_header2 = "x-ctyun";

// V4 add
static const char* specify_canon_header_v4_host = "host";
static const char* specify_canon_header_v4_contenttype = "content-type";
static const char* specify_canon_header_v4_contentlength = "content-length";
static const char* specify_canon_header_v4_x_amz = "x-amz";



static char * trim(char * src);
static int exist(const char** src,int num, const char* target);
static void split_kv(const char* src, const char* delim, key_value* kv);
static void build_query_kv_map(const char* query_args, rb_node_t** root);
static void encode_kv_map(rb_node_t* root, int need_code, int sign, char* out_str);
static void build_header_kv_map(const char* headers, rb_node_t** root);
static void canon_headers(const rb_node_t* root, char* out_headers);
static void canon_resource(const char* bucket, const char* obj,
    const char* query_args, char* res);

bool isContained(const char *str, char c)
{
    if (str == NULL)
    {
        return false;
    }
    const char *p = str;
    while (*p != '\0')
    {
        if (*p == c)
        {
            return true;
        }
        p++;
    }
    return false;
}

//*save_ptr等价于以前的静态指针
char *mystrtok_r(char* string_org,const char* demial, char **save_ptr)
{
    char *str = NULL;         //返回的字符串
    const char *ctrl = demial; //分隔符
    
    
    if (demial == NULL)
    {
        return NULL;
    }
    
    
    if (string_org == NULL && *save_ptr == NULL)
    {
        return NULL;
    }
    
    
    //将分隔符放入map中
    char map[255] = {0};
    size_t len = 0;
    while (*ctrl != '\0')
    {
        if (isContained(map, *ctrl))
        {
            continue;
        }
        map[len] = *ctrl;
        len++;
        ctrl++;
    }
    
    
    if (string_org == NULL)
    {
        str = *save_ptr;
    }
    else
    {
        str = string_org;
    }
    
    
    //忽略掉字符串中起始部分的分隔符,找到第一个不是分隔符的字符指针
    while(*str != '\0')
    {
        if (isContained(map, *str))
        {
            str++;
            continue;
        }
        break;
    }
    
    
    string_org = str;
    
    
    //查找第一个分隔符
    while (*str)
    {
        if (isContained(map, *str))
        {
            *str++ = '\0'; //当找到时，把匹配字符填为0，并且把str指向下一位
            break;
        }
        str++;  
    }  
    *save_ptr = str; // 把剩余字符串的指针保存到静态变量last  
    
    
    if (string_org == str)  
    {  
        *save_ptr = NULL;  
        return NULL;  
    }  
    else  
    {  
        return string_org;  
    }  
}

static char * trim(char * src) {
	int i = 0;
	char* begin = src;
	while (src[i] != '\0') {
		if (src[i] != ' ') {
			break;
		} else {
			begin++;
		}
		i++;
	}
	for (i = strlen(src) - 1; i >= 0; i--) {
		if (src[i] != ' ') {
			break;
		} else {
			src[i] = '\0';
		}
	}
	return begin;
}

static char *strlwr(char* str) {
	char *orign = str;
    if (str == NULL) {
        return NULL;
    }   
    for (; *str != '\0'; str++)
        *str = tolower(*str);
    return orign;
}

static int exist(const char** src, int num,
	const char* target) {
    int i;
    int exists = 0;
    for (i = 0; i< num; ++i) {
        if (0 == strcmp(src[i], target)) {
            exists = 1;
        }   
    }   
    return exists;
}

static void split_kv(const char* src, const char* delim, key_value* kv) {
    char* out_ptr = NULL;
	char* token = NULL;
	char* trim_token = NULL;
    char src_cpy[1024];
	int len;

    strcpy(src_cpy, src);
#ifdef _WIN32
    token = strtok_s(src_cpy, delim, &out_ptr);     
#endif
#ifdef __linux__
	token = mystrtok_r(src_cpy, delim, &out_ptr);
#endif
    if (token == NULL) {
        return;
    }
    len = strlen(token);
    kv->key = (char *) malloc (len + 1);
    strncpy(kv->key, token, len);          
    kv->key[len] = '\0';
    if (len == strlen(src)) {
        return;
    }

	token = token + len + 1;
    trim_token = trim(token);
    if (trim_token != NULL) {
        len = strlen(trim_token);
        kv->value = (char *) malloc (len + 1);
        strncpy(kv->value, trim_token, len);
        kv->value[len] = '\0';
    }
}

static int key_compare(key_value* a, key_value* b) {
    return strcmp(a->key, b->key);
}

static int key_value_compare(key_value* a, key_value* b) {
    int ret = strcmp(a->key, b->key);
    if (ret != 0) {
        return ret;
    }

    if (a->value == NULL && b->value == NULL)
        return 0;
    else if (a->value != NULL && b->value == NULL)
        return 1;
    else if (a->value == NULL && b->value != NULL)
        return -1;
    else
        return strcmp(a->value, b->value);
}

static void build_query_kv_map(const char* query_args, rb_node_t** root) {
	char* token = NULL;
	char* out_ptr = NULL;
	key_value* kv_pair = NULL;
	char src_cpy[1024] = { '\0' };
    int ret;
    strncpy(src_cpy, query_args, strlen(query_args));

	// 1. split by '&'
#ifdef _WIN32
    token = strtok_s(src_cpy, "&", &out_ptr);     
#endif
#ifdef __linux__
	token = mystrtok_r(src_cpy, "&", &out_ptr);
#endif
    while (token != NULL) {
        // 2. split by '='
        kv_pair = (key_value *) malloc (sizeof(key_value));
        kv_pair->key = NULL;
        kv_pair->value = NULL;
        // key : no need tolower and trim
        // value : need to trim, no need tolower
        split_kv(token, "=", kv_pair);
        // 3. insert key_value into rb tree
        ret = rb_insert_compare(kv_pair, *root, &key_value_compare, root);
        if (ret != 0) {
            free_kv(kv_pair);
            kv_pair = NULL;
        }
#ifdef _WIN32
        token = strtok_s(NULL, "&", &out_ptr);
#endif
#ifdef __linux__
		token = mystrtok_r(NULL, "&", &out_ptr);
#endif
    }
}
static void encode_value_V4(const char* str_source,
	char* out_str) {
	const char *in_str = str_source;
	int in_str_len = strlen(in_str);

	int idx;
	register unsigned char c;
	unsigned char const *from, *end;

	from = (unsigned char *)in_str;
	end = (unsigned char *)in_str + in_str_len;

	idx = 0;
	while (from < end) {
		c = *from++;
		if (c == ' ') {
			out_str[idx++] = '+';
		}
		/*else if (c == '/') {
			out_str[idx++] = c;
		}*/
		else if ((c < '0' && c != '-' && c != '.') ||
			(c < 'A' && c > '9') ||
			(c > 'Z' && c < 'a' && c != '_') ||
			(c > 'z')) {
			out_str[idx++] = '%';
			out_str[idx++] = hexchars[c >> 4];
			out_str[idx++] = hexchars[c & 15];
		}
		else {
			out_str[idx++] = c;
		}
	}
	out_str[idx] = '\0';
}
static void encode_value(const char* str_source,
	char* out_str) {
    const char *in_str = str_source;
    int in_str_len = strlen(in_str);

	int idx;
    register unsigned char c;
    unsigned char const *from, *end;

    from = (unsigned char *)in_str;
    end = (unsigned char *)in_str + in_str_len;

    idx = 0;
    while (from < end) {
        c = *from++;
        if (c == ' ') {
            out_str[idx++] = '+';
        } else if (c == '/') {
            out_str[idx++] = c;
        }else if ((c < '0' && c != '-' && c != '.') ||
               (c < 'A' && c > '9') ||
                (c > 'Z' && c < 'a' && c != '_') ||
                (c > 'z')) {
            out_str[idx++] = '%';
            out_str[idx++] = hexchars[c >> 4];
            out_str[idx++] = hexchars[c & 15];
        } else {
            out_str[idx++] = c;
        }
    }
    out_str[idx] = '\0';
}

static void encode_kv_map(rb_node_t* root, int need_code, int sign, char* out_str) {
	key_value* cur_kv = NULL;
	char* coded_value = NULL;

	if (root == NULL) {
		return;
	}
	if (root->left != NULL) {
		encode_kv_map(root->left, need_code, sign, out_str);
	}
	// encode
	cur_kv = root->kv;
	if (!sign || exist(SUB_RESOURCES, num_of_sub, cur_kv->key)
		|| exist(RESPONSE_OVERIDES, num_of_rides, cur_kv->key)) {
	    // join key=coded_value with &
        strcat(out_str, cur_kv->key);
        if (cur_kv->value != NULL) {
            strcat(out_str, "=");
            if (need_code) {
                int size = strlen(cur_kv->value) * 3;                                              
                coded_value = (char *) malloc (size);;
                memset(coded_value, '\0', size);
                encode_value(cur_kv->value, coded_value);
                strcat(out_str, coded_value);
                free(coded_value);
            } else {
                strcat(out_str, cur_kv->value);
            }
        }
        strcat(out_str, "&");
    }
	if (root->right != NULL) {
		encode_kv_map(root->right, need_code, sign, out_str);
	}
}
static void encode_kv_map_v4(rb_node_t* root, int need_code, int sign, char* out_str) {
    key_value* cur_kv = NULL;
    char* coded_value = NULL;

    if (root == NULL) {
        return;
    }
    if (root->left != NULL) {
        encode_kv_map_v4(root->left, need_code, sign, out_str);
    }
    // encode
    cur_kv = root->kv;
    if (!sign || exist(SUB_RESOURCES_V4, num_of_sub_v4, cur_kv->key)
            || exist(RESPONSE_OVERIDES, num_of_rides, cur_kv->key)) {
            // join key=coded_value with &
            strcat(out_str, cur_kv->key);
        strcat(out_str, "=");
        if (cur_kv->value != NULL) {
            if (need_code) {
                int size = strlen(cur_kv->value) * 3;
                coded_value = (char *) malloc (size);;
                memset(coded_value, '\0', size);
				encode_value_V4(cur_kv->value, coded_value);
                strcat(out_str, coded_value);
                free(coded_value);
            } else {
                strcat(out_str, cur_kv->value);
            }
        }
        strcat(out_str, "&");
    }
    if (root->right != NULL) {
        encode_kv_map_v4(root->right, need_code, sign, out_str);
    }
}
static void encode_kv_map_v4_presigned_url(rb_node_t* root, int need_code, int sign, char* out_str) {
    key_value* cur_kv = NULL;
    char* coded_value = NULL;

    if (root == NULL) {
        return;
    }
    if (root->left != NULL) {
        encode_kv_map_v4_presigned_url(root->left, need_code, sign, out_str);
    }
    // encode
    cur_kv = root->kv;
    if (sign) {
        // join key=coded_value with &
        strcat(out_str, cur_kv->key);
        strcat(out_str, "=");
        if (cur_kv->value != NULL) {
            if (need_code) {
                int size = strlen(cur_kv->value) * 3;
                coded_value = (char *) malloc (size);;
                memset(coded_value, '\0', size);
                //encode_value(cur_kv->value, coded_value);
                urlEncode_presigned_url(coded_value, cur_kv->value, size);
                strcat(out_str, coded_value);
                free(coded_value);
            } else {
                strcat(out_str, cur_kv->value);
            }
        }
        strcat(out_str, "&");
    }
    if (root->right != NULL) {
        encode_kv_map_v4_presigned_url(root->right, need_code, sign, out_str);
    }
}

char* make_resource_quote(const char* query_args, char* res) {
	int len;
	rb_node_t* root = NULL;
	if (query_args == NULL || strlen(query_args) <= 0) {
		return res;
	}
	// 1. build query_kv map
	build_query_kv_map(query_args, &root);
	// 2. iterate query_kv map and encode value
	encode_kv_map(root, 1, 0, res);
	len = strlen(res);
	if (len > 0) {
		len = len - 1;
		memset(res + len, '\0', 1);
	}
	// 3. free kv map
	rb_erase_all(root);
	return res;
}

static void canon_resource(const char* bucket,
	const char* obj, const char* query_args, char* res) {
    int len;
    rb_node_t* root = NULL;
    char new_query_args[1024] = { '\0' };

    strcat(res, "/");
    if (bucket != NULL) {
        strcat(res, bucket);
        
        if (obj != NULL) {
            strcat(res, "/");
            strcat(res, obj);
        }
    }

    if (query_args != NULL
		&& strlen(query_args) > 0) {
        // 1. build query_kv map
        build_query_kv_map(query_args, &root);
		// 2. iterate query_kv map and encode value
        encode_kv_map(root, 0, 1, new_query_args);
        len = strlen(new_query_args);
        if (len > 0) {
            len = len -1;
            memset(new_query_args + len, '\0', 1);
        }
		// 3. free kv map
		rb_erase_all(root);
        if (len <= 0) {
            return;
        }
        // 4. append to the tail of res
        strcat(res, "?");
        strcat(res, new_query_args);
    }
}

static void build_header_kv_map(const char* headers,
	rb_node_t** root) {
	char* token = NULL;
	char* out_ptr = NULL;
	key_value* kv_pair = NULL;
	char src_cpy[1024] = { '\0' };
    int ret;
    strncpy(src_cpy, headers, strlen(headers));

	// 1. split by '\n'
#ifdef _WIN32
    token = strtok_s(src_cpy, "\n", &out_ptr);     
#endif
#ifdef __linux__
	token = mystrtok_r(src_cpy, "\n", &out_ptr);
#endif
    while (token != NULL) {
        // 2. split by ':'
        kv_pair = (key_value *) malloc (sizeof(key_value));        
        kv_pair->key = NULL;
        kv_pair->value = NULL;
		// key : need tolower, no need to trim
        // value : need to trim, no need tolower
        split_kv(token, ":", kv_pair);
		strlwr(kv_pair->key);
        // 3. insert key_value into rb tree
        ret = rb_insert_compare(kv_pair, *root, &key_compare, root);
        if (ret != 0) {
            free_kv(kv_pair);
            kv_pair = NULL;
        }
#ifdef _WIN32
        token = strtok_s(NULL, "\n", &out_ptr);
#endif
#ifdef __linux__
		token = mystrtok_r(NULL, "\n", &out_ptr);
#endif
    }
}
static void make_canon_headers_v4(const rb_node_t* root, char* out_headers) {

    key_value* cur_kv = NULL;
    if (root == NULL) {
        return;
    }
    if (root->left != NULL) {
        make_canon_headers_v4(root->left, out_headers);
    }
    cur_kv = root->kv;
    // compare key with specify_header and join kv pairs with '\n'
    if (strncmp(cur_kv->key, specify_canon_header_v4_host, strlen(specify_canon_header_v4_host)) == 0
        || strncmp(cur_kv->key, specify_canon_header_v4_contenttype, strlen(specify_canon_header_v4_contenttype)) == 0
        || strncmp(cur_kv->key, specify_canon_header_v4_contentlength, strlen(specify_canon_header_v4_contentlength)) == 0
        || strncmp(cur_kv->key, specify_canon_header_v4_x_amz, strlen(specify_canon_header_v4_x_amz)) == 0) {
        strcat(out_headers, cur_kv->key);
        if (cur_kv->value != NULL) {
            strcat(out_headers, ":");
            strcat(out_headers, cur_kv->value);
            strcat(out_headers, "\n");
        }
    }

    if (root->right != NULL) {
        make_canon_headers_v4(root->right, out_headers);
    }
}

static void make_signed_headers_v4(const rb_node_t* root, char* out_headers) {

    key_value* cur_kv = NULL;
    if (root == NULL) {
        return;
    }
    if (root->left != NULL) {
        make_signed_headers_v4(root->left, out_headers);
    }
    cur_kv = root->kv;
    // compare key with specify_header and join kv pairs with '\n'
    if (strncmp(cur_kv->key, specify_canon_header_v4_host, strlen(specify_canon_header_v4_host)) == 0
        || strncmp(cur_kv->key, specify_canon_header_v4_contenttype, strlen(specify_canon_header_v4_contenttype)) == 0
        || strncmp(cur_kv->key, specify_canon_header_v4_contentlength, strlen(specify_canon_header_v4_contentlength)) == 0
        || strncmp(cur_kv->key, specify_canon_header_v4_x_amz, strlen(specify_canon_header_v4_x_amz)) == 0) {
        if(strlen(out_headers) > 0){
            strcat(out_headers, ";");
        }
        strcat(out_headers, cur_kv->key);
    }

    if (root->right != NULL) {
        make_signed_headers_v4(root->right, out_headers);
    }
}

extern void make_canon_query_param_V4( const char* query_args, char* res) {

    int len;
    rb_node_t* root = NULL;
    char new_query_args[1024] = { '\0' };

    if (query_args != NULL
        && strlen(query_args) > 0) {
        // 1. build query_kv map
        build_query_kv_map(query_args, &root);
        // 2. iterate query_kv map and encode value
        encode_kv_map_v4(root, 1, 1, new_query_args);
        len = strlen(new_query_args);
        if (len > 0) {
            len = len -1;
            memset(new_query_args + len, '\0', 1);
        }
        // 3. free kv map
        rb_erase_all(root);
        if (len <= 0) {
            return;
        }
        // 4. append to the tail of res
        //strcat(res, "?");
        strcat(res, new_query_args);
    }
}

extern void make_canon_query_param_V4_presigned_url( const char* query_args, char* res) {

    int len;
    rb_node_t* root = NULL;
    char new_query_args[1024] = { '\0' };

    if (query_args != NULL
        && strlen(query_args) > 0) {
        // 1. build query_kv map
        build_query_kv_map(query_args, &root);
        // 2. iterate query_kv map and encode value
        encode_kv_map_v4_presigned_url(root, 1, 1, new_query_args);
        len = strlen(new_query_args);
        if (len > 0) {
            len = len -1;
            memset(new_query_args + len, '\0', 1);
        }
        // 3. free kv map
        rb_erase_all(root);
        if (len <= 0) {
            return;
        }
        // 4. append to the tail of res
        //strcat(res, "?");
        strcat(res, new_query_args);
    }
}

extern char* make_string_to_sign_V4(const char* utc_time, const char* gmt_time,  const char* canon_request,
                                    const char* region, const char* service, char* string_to_sign){

   // 1 ASW
   strcat(string_to_sign, "AWS4-HMAC-SHA256");
   strcat(string_to_sign, "\n");

   // 2 timestamp
   strcat(string_to_sign, utc_time);
   strcat(string_to_sign, "\n");

   // 3 scope
   char scope[512+1] = {"\0"};
   strcat(scope, gmt_time);
   strcat(scope, "/");
   strcat(scope, region);
   strcat(scope, "/");
   strcat(scope, service);
   strcat(scope, "/aws4_request");

   strcat(string_to_sign, scope);
   strcat(string_to_sign, "\n");

   // 4 hex sha256 canon request
   char sha256_canon_request[32+1] = {"\0"};
   sha2((unsigned char*)canon_request, strlen(canon_request), (unsigned char*)sha256_canon_request, 0);
   char hex_sha256_canon_request[64+1] = {"\0"};
   HexToStr(hex_sha256_canon_request, sha256_canon_request, 32);
   strcat(string_to_sign, hex_sha256_canon_request);

   return string_to_sign;
}

extern char* make_canonicalized_request_V4(int method_type, const char* bucket, const char* object,
                                          const char* query_args,const char* headers, char* canon_request, char* signed_headers)
{
    // build header map
    struct rb_node_t* root = NULL;
    if (headers != NULL) {
        build_header_kv_map(headers, &root);
    }
    // 1 Http Verb
    static const char* method[MethodType_END] = {"GET", "PUT", "DELETE", "POST", "HEAD"};
    strcat(canon_request, method[method_type]);
    strcat(canon_request, "\n");

    // 2 canonical URI
    char encode_bucket[256+1] = {"\0"};
    char encode_object[256+1] = {"\0"};

    char canonical_url[256] = { '\0' };
    if (NULL != bucket){
        urlEncode(encode_bucket, bucket, strlen(bucket));
        strcat(canonical_url, "/");
        strcat(canonical_url, encode_bucket);
    }
    if (NULL != object){
        urlEncode(encode_object, object, strlen(object));
        strcat(canonical_url, "/");
        strcat(canonical_url, encode_object);
    }
    if(0 == strlen(canonical_url)){
        strcat(canonical_url, "/");
    }
    strcat(canon_request,canonical_url);
    strcat(canon_request, "\n");


    // 3. canonicalizedHeader query_args
    char canned_query_args[1024] = { '\0' };
    make_canon_query_param_V4(query_args, canned_query_args);
    strcat(canon_request, canned_query_args);
    strcat(canon_request, "\n");

    // 4 make canonicalizedHeader
    char canned_headers[1024] = { '\0' };
    make_canon_headers_v4(root, canned_headers);
    strcat(canon_request, canned_headers);
    strcat(canon_request, "\n");

    // 5. make SignedHeaders
    char sign_headers[1024] = { '\0' };
    make_signed_headers_v4(root, sign_headers);
    strcat(canon_request, sign_headers);
    strcpy(signed_headers, sign_headers);
    strcat(canon_request, "\n");

    // Hex hash payload
    key_value kv;
    rb_node_t* sha256_content = NULL;
    kv.key = "x-amz-content-sha256";
    int ret = rb_search_compare(&kv, root, &key_compare, &sha256_content);
    if (ret == 0 && sha256_content != NULL
        && sha256_content->kv != NULL && sha256_content->kv->value != NULL) {
        if (strcmp(sha256_content->kv->value, "UNSIGNED-PAYLOAD") == 0 ){
            //strcat(canon_request, "e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855");
            strcat(canon_request , "UNSIGNED-PAYLOAD");
        }else{
            //strcat(canon_request, "\n");
            strcat(canon_request, sha256_content->kv->value);
        }
    }
    return canon_request;
}
extern char* make_canonicalized_request_V4_presigned_url(int method_type, const char* bucket, const char* object,
                                          const char* query_args,const char* headers, char* canon_request, char* signed_headers)
{
    // build header map
    struct rb_node_t* root = NULL;
    if (headers != NULL) {
        build_header_kv_map(headers, &root);
    }
    // 1 Http Verb
    static const char* method[MethodType_END] = {"GET", "PUT", "DELETE", "POST", "HEAD"};
    strcat(canon_request, method[method_type]);
    strcat(canon_request, "\n");

    // 2 canonical URI
    char encode_bucket[256+1] = {"\0"};
    char encode_object[256+1] = {"\0"};

    char canonical_url[256] = { '\0' };
    if (NULL != bucket){
        urlEncode(encode_bucket, bucket, strlen(bucket));
        strcat(canonical_url, "/");
        strcat(canonical_url, encode_bucket);
    }
    if (NULL != object){
        urlEncode(encode_object, object, strlen(object));
        strcat(canonical_url, "/");
        strcat(canonical_url, encode_object);
    }
    if(0 == strlen(canonical_url)){
        strcat(canonical_url, "/");
    }
    strcat(canon_request,canonical_url);
    strcat(canon_request, "\n");

    // 3. make canonicalizedHeader
    char canned_headers[1024] = { '\0' };
    make_canon_headers_v4(root, canned_headers);

    // 4. make SignedHeaders
    char sign_headers[1024] = { '\0' };
    make_signed_headers_v4(root, sign_headers);


    // 5. canonicalizedHeader query_args
    strcat(query_args, "&X-Amz-SignedHeaders=");
    strcat(query_args, sign_headers);
    char canned_query_args[1024] = { '\0' };
    make_canon_query_param_V4_presigned_url(query_args, canned_query_args);
    strcat(canon_request, canned_query_args);
    strcat(canon_request, "\n");

    strcat(canon_request, canned_headers);
    strcat(canon_request, "\n");

    strcat(canon_request, sign_headers);
    strcpy(signed_headers, sign_headers);
    strcat(canon_request, "\n");


    // Hex hash payload
    key_value kv;
    rb_node_t* sha256_content = NULL;
    kv.key = "x-amz-content-sha256";
    int ret = rb_search_compare(&kv, root, &key_compare, &sha256_content);
    if (ret == 0 && sha256_content != NULL
        && sha256_content->kv != NULL && sha256_content->kv->value != NULL) {
        if (strcmp(sha256_content->kv->value, "UNSIGNED-PAYLOAD") == 0 ){
            //strcat(canon_request, "e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855");
            strcat(canon_request , "UNSIGNED-PAYLOAD");
        }else{
            //strcat(canon_request, "\n");
            strcat(canon_request, sha256_content->kv->value);
        }
    }
    else{
         strcat(canon_request , "UNSIGNED-PAYLOAD");
    }
    return canon_request;
}

static void canon_headers(const rb_node_t* root,
        char* out_headers) {
	key_value* cur_kv = NULL;
	if (root == NULL) {
		return;
	}
	if (root->left != NULL) {
		canon_headers(root->left, out_headers);
	}
	cur_kv = root->kv;
	// compare key with specify_header and join kv pairs with '\n'
	if (strncmp(cur_kv->key, specify_header, strlen(specify_header)) == 0) {
		strcat(out_headers, cur_kv->key);
		if (cur_kv->value != NULL) {
            strcat(out_headers, ":");
            strcat(out_headers, cur_kv->value);
            strcat(out_headers, "\n");
		}
	}
//    if (strncmp(cur_kv->key, specify_header2, strlen(specify_header2)) == 0) {
//        strcat(out_headers, cur_kv->key);
//        if (cur_kv->value != NULL) {
//            strcat(out_headers, ":");
//            strcat(out_headers, cur_kv->value);
//            strcat(out_headers, "\n");
//        }
//    }
	if (root->right != NULL) {
		canon_headers(root->right, out_headers);
	}
}

extern char* make_origin_sign(int method_type,
	const char* bucket, const char* obj,
	const char* query_args, const char* headers,
	const char* time, char* origin_sign) {

	static const char* method[MethodType_END] = {"GET", "PUT", "DELETE", "POST", "HEAD"};
	struct rb_node_t* root = NULL;
	key_value kv;
	rb_node_t* md5_node = NULL;
	rb_node_t* type_node = NULL;
    int ret;
	char canned_headers[1024] = { '\0' };
    
	// 1. append: ${method}\n
    strcpy(origin_sign, method[method_type]);
    strcat(origin_sign, "\n");
    // 2. build header map
	if (headers != NULL) {
		build_header_kv_map(headers, &root);
	}
    // 3. get and append: ${content-md5}\n
	kv.key = "content-md5";
    ret = rb_search_compare(&kv, root, &key_compare, &md5_node);
	if (ret == 0 && md5_node != NULL
		&& md5_node->kv != NULL && md5_node->kv->value != NULL) {
        strcat(origin_sign, md5_node->kv->value);
    }
    strcat(origin_sign, "\n");
    // 4. get and append: ${content-type}\n
	kv.key = "content-type";
    ret = rb_search_compare(&kv, root, &key_compare, &type_node);
    if (ret == 0 && type_node != NULL
		&& type_node->kv != NULL && type_node->kv->value != NULL) {
        strcat(origin_sign, type_node->kv->value);
    }
    strcat(origin_sign, "\n");
    // 5. append: ${date_time}\n
    strcat(origin_sign, time);
    strcat(origin_sign, "\n");
    // 6. canon headers, keys start with 'x-amz'
	if (headers != NULL) {
		canon_headers(root, canned_headers);
		strcat(origin_sign, canned_headers);
	}
	// 7. canon query_args
	canon_resource(bucket, obj, query_args, origin_sign);
    // 8. free header map
	rb_erase_all(root);
    printf("orign_sign:%s\n", origin_sign);
	return origin_sign;
}

extern char* get_time(char* out_time) {	
	char date[64] = { '\0' };
	time_t now = time(NULL);
	strftime(date, sizeof(date), "%a, %d %b %Y %H:%M:%S GMT", gmtime(&now));
	strncpy(out_time, date, sizeof(date));
	return out_time;
}
extern char* get_date(char* out_time) {
    char date[64] = { '\0' };
    time_t now = time(NULL);
    strftime(date, sizeof(date), "%Y%m%d", gmtime(&now));
    strncpy(out_time, date, sizeof(date));
    return out_time;
}
extern char* get_utc_time(char* out_time) {
    char date[64] = { '\0' };
    time_t now = time(NULL);
    strftime(date, sizeof(date), "%Y%m%dT%H%M%SZ", gmtime(&now));
    strncpy(out_time, date, sizeof(date));
    return out_time;
}

extern char* get_utc_time_expires(char* out_time, const int expire_sec) {
    char date[64] = { '\0' };
    time_t now = time(NULL);
    now += expire_sec;
    sprintf(out_time, "%d", now);
    //strftime(date, sizeof(date), "%Y%m%dT%H%M%SZ", gmtime(&now));
    //strncpy(out_time, date, sizeof(date));
    return out_time;
}

extern void get_region(const char* body, const char* host, char* region){
    if(NULL == host || strlen(host) <= 0){
        return ;
    }

    if(NULL != body && (
            strstr(body, "Action=CreateAccessKey") != NULL
            || strstr(body, "Action=DeleteAccessKey") != NULL
            || strstr(body, "Action=UpdateAccessKey") != NULL
            || strstr(body, "Action=ListAccessKey") != NULL)
            ){
        int len = strlen(host) - strlen("oos--iam.ctyunapi.cn");
        if (len > 0){
             strncpy(region, host + strlen("oos-"), len);
        }
    }else{
        int len = strlen(host) - strlen("oos-.ctyunapi.cn");
        if(len > 0){
            strncpy(region, host + strlen("oos-"), len);
        }
    }
}



extern void get_service(const char* body, char* service){
    if(NULL != body && (
            strstr(body, "Action=CreateAccessKey") != NULL
            || strstr(body, "Action=DeleteAccessKey") != NULL
            || strstr(body, "Action=UpdateAccessKey") != NULL
            || strstr(body, "Action=ListAccessKey") != NULL)
            ){
        strcpy(service, "sts");
    }else{
        strcpy(service, "s3");
    }
}







